@@ -23,6 +23,132 @@ static const char LM_MASKED_PLAIN[] = "[notfound]";
  * algo specific
  */
 
+// original headers from luks.h
+
+#define LUKS_CIPHERNAME_L 32
+#define LUKS_CIPHERMODE_L 32
+#define LUKS_HASHSPEC_L 32
+#define LUKS_DIGESTSIZE 20 // since SHA1
+#define LUKS_HMACSIZE 32
+#define LUKS_SALTSIZE 32
+#define LUKS_NUMKEYS 8
+// Minimal number of iterations
+#define LUKS_MKD_ITERATIONS_MIN  1000
+#define LUKS_SLOT_ITERATIONS_MIN 1000
+#define LUKS_KEY_DISABLED_OLD 0
+#define LUKS_KEY_ENABLED_OLD 0xCAFE
+#define LUKS_KEY_DISABLED 0x0000DEAD
+#define LUKS_KEY_ENABLED  0x00AC71F3
+#define LUKS_STRIPES 4000
+// partition header starts with magic
+#define LUKS_MAGIC {'L','U','K','S', 0xba, 0xbe};
+#define LUKS_MAGIC_L 6
+/* Actually we need only 37, but we don't want struct autoaligning to kick in */
+#define UUID_STRING_L 40
+/* Offset to keyslot area [in bytes] */
+#define LUKS_ALIGN_KEYSLOTS 4096
+
+struct luks_phdr {
+  char      magic[LUKS_MAGIC_L];
+  uint16_t  version;
+  char      cipherName[LUKS_CIPHERNAME_L];
+  char      cipherMode[LUKS_CIPHERMODE_L];
+  char      hashSpec[LUKS_HASHSPEC_L];
+  uint32_t  payloadOffset;
+  uint32_t  keyBytes;
+  char      mkDigest[LUKS_DIGESTSIZE];
+  char      mkDigestSalt[LUKS_SALTSIZE];
+  uint32_t  mkDigestIterations;
+  char      uuid[UUID_STRING_L];
+  struct {
+    uint32_t active;
+    /* parameters used for password processing */
+    uint32_t passwordIterations;
+    char     passwordSalt[LUKS_SALTSIZE];
+    /* parameters used for AF store/load */
+    uint32_t keyMaterialOffset;
+    uint32_t stripes;
+  } keyblock[LUKS_NUMKEYS];
+  /* Align it to 512 sector size */
+  char       _padding[432];
+};
+
+// not from original headers start with hc_
+
+typedef enum hc_luks_hash_type
+{
+  HC_LUKS_HASH_TYPE_SHA1      = 1,
+  HC_LUKS_HASH_TYPE_SHA256    = 2,
+  HC_LUKS_HASH_TYPE_SHA512    = 3,
+  HC_LUKS_HASH_TYPE_RIPEMD160 = 4,
+  HC_LUKS_HASH_TYPE_WHIRLPOOL = 5,
+
+} hc_luks_hash_type_t;
+
+typedef enum hc_luks_key_size
+{
+  HC_LUKS_KEY_SIZE_128 = 128,
+  HC_LUKS_KEY_SIZE_256 = 256,
+  HC_LUKS_KEY_SIZE_512 = 512,
+
+} hc_luks_key_size_t;
+
+typedef enum hc_luks_cipher_type
+{
+  HC_LUKS_CIPHER_TYPE_AES     = 1,
+  HC_LUKS_CIPHER_TYPE_SERPENT = 2,
+  HC_LUKS_CIPHER_TYPE_TWOFISH = 3,
+
+} hc_luks_cipher_type_t;
+
+typedef enum hc_luks_cipher_mode
+{
+  HC_LUKS_CIPHER_MODE_CBC_ESSIV = 1,
+  HC_LUKS_CIPHER_MODE_CBC_PLAIN = 2,
+  HC_LUKS_CIPHER_MODE_XTS_PLAIN = 3,
+
+} hc_luks_cipher_mode_t;
+
+typedef struct luks
+{
+  int hash_type;    // hc_luks_hash_type_t
+  int key_size;     // hc_luks_key_size_t
+  int cipher_type;  // hc_luks_cipher_type_t
+  int cipher_mode;  // hc_luks_cipher_mode_t
+
+  u32 ct_buf[128];
+
+  u32 af_src_buf[((HC_LUKS_KEY_SIZE_512 / 8) * LUKS_STRIPES) / 4];
+
+} luks_t;
+
+typedef struct itunes_backup
+{
+  u32 wpky[10];
+  u32 dpsl[5];
+
+} itunes_backup_t;
+
+typedef struct blake2
+{
+  u64 h[8];
+  u64 t[2];
+  u64 f[2];
+  u32 buflen;
+  u32 outlen;
+  u8  last_node;
+
+} blake2_t;
+
+typedef struct chacha20
+{
+  u32 iv[2];
+  u32 plain[2];
+  u32 position[2];
+  u32 offset;
+
+} chacha20_t;
+
 typedef struct rar5
 {
   u32 iv[4];
@@ -53,14 +179,21 @@ typedef struct pdf
 typedef struct wpa
 {
   u32  pke[25];
-  u32  eapol[64];
-  int  eapol_size;
-  int  keyver;
-  u8   orig_mac1[6];
-  u8   orig_mac2[6];
-  u8   orig_nonce1[32];
-  u8   orig_nonce2[32];
-  int  essid_reuse;
+  u32  eapol[64 + 16];
+  u16  eapol_len;
+  u8   message_pair;
+  int  message_pair_chgd;
+  u8   keyver;
+  u8   orig_mac_ap[6];
+  u8   orig_mac_sta[6];
+  u8   orig_nonce_ap[32];
+  u8   orig_nonce_sta[32];
+  u8   essid_len;
+  u8   essid[32];
+  u32  keymic[4];
+  u32  hash[4];
+  int  nonce_compare;
+  int  nonce_error_corrections;
 
 } wpa_t;
 
@@ -291,6 +424,68 @@ typedef struct psafe3
 
 } psafe3_t;
 
+typedef struct dpapimk
+{
+  u32 version;
+  u32 context;
+
+  u32 SID[32];
+  u32 SID_len;
+  u32 SID_offset;
+
+  /* here only for possible
+     forward compatibiliy
+  */
+  // u8 cipher_algo[16];
+  // u8 hash_algo[16];
+
+  u32 iv[4];
+  u32 contents_len;
+  u32 contents[128];
+
+} dpapimk_t;
+
+typedef struct jks_sha1
+{
+  u32 checksum[5];
+  u32 iv[5];
+  u32 enc_key_buf[4096];
+  u32 enc_key_len;
+  u32 der[5];
+  u32 alias[16];
+
+} jks_sha1_t;
+
+typedef struct ethereum_pbkdf2
+{
+  u32 salt_buf[16];
+  u32 ciphertext[8];
+
+} ethereum_pbkdf2_t;
+
+typedef struct ethereum_scrypt
+{
+  u32 salt_buf[16];
+  u32 ciphertext[8];
+
+} ethereum_scrypt_t;
+
+typedef struct luks_tmp
+{
+  u32 ipad32[8];
+  u64 ipad64[8];
+
+  u32 opad32[8];
+  u64 opad64[8];
+
+  u32 dgst32[32];
+  u64 dgst64[16];
+
+  u32 out32[32];
+  u64 out64[16];
+
+} luks_tmp_t;
+
 typedef struct pdf14_tmp
 {
   u32 digest[4];
@@ -538,6 +733,17 @@ typedef struct saph_sha1_tmp
 
 } saph_sha1_tmp_t;
 
+typedef struct pbkdf1_sha1_tmp
+{
+  // pbkdf1-sha1 is limited to 160 bits
+
+  u32  ipad[5];
+  u32  opad[5];
+
+  u32  out[5];
+
+} pbkdf1_sha1_tmp_t;
+
 typedef struct pbkdf2_md5_tmp
 {
   u32  ipad[4];
@@ -626,7 +832,7 @@ typedef struct cram_md5
 
 } cram_md5_t;
 
-typedef struct seven_zip
+typedef struct seven_zip_hook_salt
 {
   u32 iv_buf[4];
   u32 iv_len;
@@ -635,13 +841,21 @@ typedef struct seven_zip
   u32 salt_len;
 
   u32 crc;
+  u32 crc_len;
 
-  u32 data_buf[96];
+  u8  data_type;
+
+  u32 data_buf[81882];
   u32 data_len;
 
   u32 unpack_size;
 
-} seven_zip_t;
+  char coder_attributes[5 + 1];
+  u8   coder_attributes_len;
+
+  int aes_len; // pre-computed length of the maximal (subset of) data we need for AES-CBC
+
+} seven_zip_hook_salt_t;
 
 typedef struct axcrypt_tmp
 {
@@ -657,6 +871,32 @@ typedef struct keepass_tmp
 
 } keepass_tmp_t;
 
+typedef struct dpapimk_tmp
+{
+  /* dedicated to hmac-sha1 */
+  u32 ipad[5];
+  u32 opad[5];
+  u32 dgst[10];
+  u32 out[10];
+
+  u32 userKey[5];
+
+  /* dedicated to hmac-sha512 */
+  u64 ipad64[8];
+  u64 opad64[8];
+  u64 dgst64[16];
+  u64 out64[16];
+
+} dpapimk_tmp_t;
+
+typedef struct seven_zip_hook
+{
+  u32 ukey[8];
+
+  u32 hook_success;
+
+} seven_zip_hook_t;
+
 typedef struct struct_psafe2_hdr
 {
   u32  random[2];
@@ -666,22 +906,44 @@ typedef struct struct_psafe2_hdr
 
 } psafe2_hdr;
 
-typedef struct hccap
+typedef enum
 {
-  char essid[36];
+  MESSAGE_PAIR_M12E2 = 0,
+  MESSAGE_PAIR_M14E4 = 1,
+  MESSAGE_PAIR_M32E2 = 2,
+  MESSAGE_PAIR_M32E3 = 3,
+  MESSAGE_PAIR_M34E3 = 4,
+  MESSAGE_PAIR_M34E4 = 5,
 
-  u8   mac1[6];
-  u8   mac2[6];
-  u8   nonce1[32];
-  u8   nonce2[32];
+} message_pair_t;
 
-  u8   eapol[256];
-  int  eapol_size;
+#define HCCAPX_VERSION   4
+#define HCCAPX_SIGNATURE 0x58504348 // HCPX
 
-  int  keyver;
-  u8   keymic[16];
+// this is required to force mingw to accept the packed attribute
+#pragma pack(push,1)
 
-} hccap_t;
+struct hccapx
+{
+  u32 signature;
+  u32 version;
+  u8  message_pair;
+  u8  essid_len;
+  u8  essid[32];
+  u8  keyver;
+  u8  keymic[16];
+  u8  mac_ap[6];
+  u8  nonce_ap[32];
+  u8  mac_sta[6];
+  u8  nonce_sta[32];
+  u16 eapol_len;
+  u8  eapol[256];
+
+} __attribute__((packed));
+
+typedef struct hccapx hccapx_t;
+
+#pragma pack(pop)
 
 /**
  * hashtypes enums
@@ -727,6 +989,8 @@ typedef enum display_len
   DISPLAY_LEN_MIN_501   = 104,
   DISPLAY_LEN_MAX_500   = 3 + 1 + 8 + 22,
   DISPLAY_LEN_MAX_501   = 104,
+  DISPLAY_LEN_MIN_600   = 8 + 128,
+  DISPLAY_LEN_MAX_600   = 8 + 128,
   DISPLAY_LEN_MIN_900   = 32,
   DISPLAY_LEN_MAX_900   = 32,
   DISPLAY_LEN_MIN_910   = 32 + 1 + 0,
@@ -805,10 +1069,12 @@ typedef enum display_len
   DISPLAY_LEN_MAX_3100H = 16 + 1 + 60,
   DISPLAY_LEN_MIN_3200  = 60,
   DISPLAY_LEN_MAX_3200  = 60,
-  DISPLAY_LEN_MIN_3711  = 3 +  0 + 1 + 32,
-  DISPLAY_LEN_MAX_3711  = 3 + 31 + 1 + 32,
   DISPLAY_LEN_MIN_4300  = 32,
   DISPLAY_LEN_MAX_4300  = 32,
+  DISPLAY_LEN_MIN_4500  = 40,
+  DISPLAY_LEN_MAX_4500  = 40,
+  DISPLAY_LEN_MIN_4520  = 40 + 1 + 1,
+  DISPLAY_LEN_MAX_4520  = 40 + 1 + 64,
   DISPLAY_LEN_MIN_4800  = 32 + 1 + 32 + 1 + 2,
   DISPLAY_LEN_MAX_4800  = 32 + 1 + 32 + 1 + 2,
   DISPLAY_LEN_MIN_5000  = 16,
@@ -845,6 +1111,8 @@ typedef enum display_len
   DISPLAY_LEN_MAX_6800  = 32 + 1 + 5 + 1 + 32,
   DISPLAY_LEN_MIN_6900  = 64,
   DISPLAY_LEN_MAX_6900  = 64,
+  DISPLAY_LEN_MIN_7000  = 3 + 44,
+  DISPLAY_LEN_MAX_7000  = 3 + 44,
   DISPLAY_LEN_MIN_7100  =  4 + 2 + 1 + 64 + 1 + 128,
   DISPLAY_LEN_MAX_7100  =  4 + 5 + 1 + 64 + 1 + 128,
   DISPLAY_LEN_MIN_7200  = 19 + 1 + 1 +   1 + 128,
@@ -939,8 +1207,8 @@ typedef enum display_len
   DISPLAY_LEN_MAX_11400 = 6 + 512 + 1 + 512 + 1 + 116 + 1 + 116 + 1 + 246 + 1 + 245 + 1 + 246 + 1 + 245 + 1 + 50 + 1 + 50 + 1 + 50 + 1 + 50 + 1 + 3 + 1 + 32,
   DISPLAY_LEN_MIN_11500 = 8 + 1 + 8,
   DISPLAY_LEN_MAX_11500 = 8 + 1 + 8,
-  DISPLAY_LEN_MIN_11600 = 1 + 2 + 1 + 1 + 1 + 1 + 1 + 1 + 1 +  0 + 1 + 1 + 1 + 32 + 1 +  1 + 1 + 1 + 1 + 1 + 1 +   2,
-  DISPLAY_LEN_MAX_11600 = 1 + 2 + 1 + 1 + 1 + 2 + 1 + 1 + 1 + 64 + 1 + 1 + 1 + 32 + 1 + 10 + 1 + 3 + 1 + 3 + 1 + 768,
+  DISPLAY_LEN_MIN_11600 = 1 + 2 + 1 + 1 + 1 + 1 + 1 + 1 + 1 +  0 + 1 + 1 + 1 + 32 + 1 +  1 + 1 + 1 + 1 + 1 + 1 +      2 + 0 + 0 + 0 +  0,
+  DISPLAY_LEN_MAX_11600 = 1 + 2 + 1 + 1 + 1 + 2 + 1 + 1 + 1 + 64 + 1 + 1 + 1 + 32 + 1 + 10 + 1 + 4 + 1 + 4 + 1 + 655056 + 1 + 4 + 1 + 10,
   DISPLAY_LEN_MIN_11700 = 64,
   DISPLAY_LEN_MAX_11700 = 64,
   DISPLAY_LEN_MIN_11800 = 128,
@@ -991,6 +1259,26 @@ typedef enum display_len
   DISPLAY_LEN_MAX_14100 = 16 + 1 + 16,
   DISPLAY_LEN_MIN_14400 = 40 + 1 + 20,
   DISPLAY_LEN_MAX_14400 = 40 + 1 + 20,
+  DISPLAY_LEN_MIN_14700 = 15 + 1 + 1 + 1 + 80 + 1 + 1 + 1 + 40 + 1,
+  DISPLAY_LEN_MAX_14700 = 15 + 1 + 2 + 1 + 80 + 1 + 6 + 1 + 40 + 1 + 9 + 1 + 40,
+  DISPLAY_LEN_MIN_14900 = 8 + 1 + 8,
+  DISPLAY_LEN_MAX_14900 = 8 + 1 + 8,
+  DISPLAY_LEN_MIN_15000 = 128 + 1 + 64,
+  DISPLAY_LEN_MAX_15000 = 128 + 1 + 64,
+  DISPLAY_LEN_MIN_15100 = 6 + 3 + 1 + 8 + 1 + 28,
+  DISPLAY_LEN_MAX_15100 = 6 + 6 + 1 + 8 + 1 + 28,
+  DISPLAY_LEN_MIN_15200 =  1 + 10 + 1 + 2 + 1 + 1 + 1 + 1 + 1 + 64,
+  DISPLAY_LEN_MAX_15200 =  1 + 10 + 1 + 2 + 1 + 8 + 1 + 5 + 1 + 20000,
+  DISPLAY_LEN_MIN_15300 =  1 + 7 + 1 + 1 + 1 + 1 + 1 +  10 + 1 + 4 + 1 + 4 + 1 +  1 + 1 + 32 + 1 + 3 + 1 + 128,
+  DISPLAY_LEN_MAX_15300 =  1 + 7 + 1 + 1 + 1 + 1 + 1 + 100 + 1 + 6 + 1 + 6 + 1 + 10 + 1 + 32 + 1 + 4 + 1 + 512,
+  DISPLAY_LEN_MIN_15400 = 10 + 1 + 16 + 1 + 1 + 1 + 16 + 1 + 16 + 1 + 16,
+  DISPLAY_LEN_MAX_15400 = 10 + 1 + 16 + 1 + 2 + 1 + 16 + 1 + 16 + 1 + 16,
+  DISPLAY_LEN_MIN_15500 = 10 + 1 + 40 + 1 + 40 + 1 +     1 + 1 + 2 + 1 + 28 + 1 +  1,
+  DISPLAY_LEN_MAX_15500 = 10 + 1 + 40 + 1 + 40 + 1 + 16384 + 1 + 2 + 1 + 28 + 1 + 64,
+  DISPLAY_LEN_MIN_15600 = 11 + 1 + 1 + 1 + 32 + 1 + 64 + 1 + 64,
+  DISPLAY_LEN_MAX_15600 = 11 + 1 + 6 + 1 + 64 + 1 + 64 + 1 + 64,
+  DISPLAY_LEN_MIN_15700 = 11 + 1 + 1 + 1 + 1 + 1 + 1 + 1 + 64 + 1 + 64 + 1 + 64,
+  DISPLAY_LEN_MAX_15700 = 11 + 1 + 6 + 1 + 1 + 1 + 1 + 1 + 64 + 1 + 64 + 1 + 64,
   DISPLAY_LEN_MIN_99999 = 1,
   DISPLAY_LEN_MAX_99999 = 55,
 
@@ -1036,6 +1324,8 @@ typedef enum display_len
   DISPLAY_LEN_MAX_133   = 28,
   DISPLAY_LEN_MIN_141   = 14 +  0 +  1 + 28,
   DISPLAY_LEN_MAX_141   = 14 + 44 +  1 + 28,
+  DISPLAY_LEN_MIN_1411  =  9 + 44 +  0,
+  DISPLAY_LEN_MAX_1411  =  9 + 44 + 68,
   DISPLAY_LEN_MIN_1441  = 14 +  0 +  1 + 43,
   DISPLAY_LEN_MAX_1441  = 14 + 24 +  1 + 43,
   DISPLAY_LEN_MIN_1711  =  9 + 86 +  0,
@@ -1056,8 +1346,14 @@ typedef enum display_len
   DISPLAY_LEN_MAX_2811  = 32 + 1 + 31,
   DISPLAY_LEN_MIN_2811H = 32 + 1 + 0,
   DISPLAY_LEN_MAX_2811H = 32 + 1 + 62,
-  DISPLAY_LEN_MIN_7600  = 40 + 1 + 32,
-  DISPLAY_LEN_MAX_7600  = 40 + 1 + 32,
+  DISPLAY_LEN_MIN_3711  = 3 +  0 + 1 + 32,
+  DISPLAY_LEN_MAX_3711  = 3 + 31 + 1 + 32,
+  DISPLAY_LEN_MIN_4521  = 40 + 1 + 32,
+  DISPLAY_LEN_MAX_4521  = 40 + 1 + 32,
+  DISPLAY_LEN_MIN_4522  = 40 + 1 + 12,
+  DISPLAY_LEN_MAX_4522  = 40 + 1 + 12,
+  DISPLAY_LEN_MIN_12001 = 9 + 64,
+  DISPLAY_LEN_MAX_12001 = 9 + 64,
 
 } display_len_t;
 
@@ -1117,171 +1413,209 @@ typedef enum hash_type
   HASH_TYPE_STDOUT              = 52,
   HASH_TYPE_DES                 = 53,
   HASH_TYPE_PLAINTEXT           = 54,
+  HASH_TYPE_LUKS                = 55,
+  HASH_TYPE_ITUNES_BACKUP_9     = 56,
+  HASH_TYPE_ITUNES_BACKUP_10    = 57,
+  HASH_TYPE_SKIP32              = 58,
+  HASH_TYPE_BLAKE2B             = 59,
+  HASH_TYPE_CHACHA20            = 60,
+  HASH_TYPE_DPAPIMK             = 61,
+  HASH_TYPE_JKS_SHA1            = 62,
 
 } hash_type_t;
 
 typedef enum kern_type
 {
-  KERN_TYPE_MD5                 = 0,
-  KERN_TYPE_MD5_PWSLT           = 10,
-  KERN_TYPE_MD5_SLTPW           = 20,
-  KERN_TYPE_MD5_PWUSLT          = 30,
-  KERN_TYPE_MD5_SLTPWU          = 40,
-  KERN_TYPE_HMACMD5_PW          = 50,
-  KERN_TYPE_HMACMD5_SLT         = 60,
-  KERN_TYPE_SHA1                = 100,
-  KERN_TYPE_SHA1_PWSLT          = 110,
-  KERN_TYPE_SHA1_SLTPW          = 120,
-  KERN_TYPE_SHA1_PWUSLT         = 130,
-  KERN_TYPE_SHA1_SLTPWU         = 140,
-  KERN_TYPE_HMACSHA1_PW         = 150,
-  KERN_TYPE_HMACSHA1_SLT        = 160,
-  KERN_TYPE_MYSQL               = 200,
-  KERN_TYPE_MYSQL41             = 300,
-  KERN_TYPE_PHPASS              = 400,
-  KERN_TYPE_MD5CRYPT            = 500,
-  KERN_TYPE_MD4                 = 900,
-  KERN_TYPE_MD4_PWU             = 1000,
-  KERN_TYPE_MD44_PWUSLT         = 1100,
-  KERN_TYPE_SHA224              = 1300,
-  KERN_TYPE_SHA256              = 1400,
-  KERN_TYPE_SHA256_PWSLT        = 1410,
-  KERN_TYPE_SHA256_SLTPW        = 1420,
-  KERN_TYPE_SHA256_PWUSLT       = 1430,
-  KERN_TYPE_SHA256_SLTPWU       = 1440,
-  KERN_TYPE_HMACSHA256_PW       = 1450,
-  KERN_TYPE_HMACSHA256_SLT      = 1460,
-  KERN_TYPE_DESCRYPT            = 1500,
-  KERN_TYPE_APR1CRYPT           = 1600,
-  KERN_TYPE_SHA512              = 1700,
-  KERN_TYPE_SHA512_PWSLT        = 1710,
-  KERN_TYPE_SHA512_SLTPW        = 1720,
-  KERN_TYPE_SHA512_PWSLTU       = 1730,
-  KERN_TYPE_SHA512_SLTPWU       = 1740,
-  KERN_TYPE_HMACSHA512_PW       = 1750,
-  KERN_TYPE_HMACSHA512_SLT      = 1760,
-  KERN_TYPE_SHA512CRYPT         = 1800,
-  KERN_TYPE_STDOUT              = 2000,
-  KERN_TYPE_DCC2                = 2100,
-  KERN_TYPE_MD5PIX              = 2400,
-  KERN_TYPE_MD5ASA              = 2410,
-  KERN_TYPE_WPA                 = 2500,
-  KERN_TYPE_MD55                = 2600,
-  KERN_TYPE_MD55_PWSLT1         = 2610,
-  KERN_TYPE_MD55_PWSLT2         = 2710,
-  KERN_TYPE_MD55_SLTPW          = 2810,
-  KERN_TYPE_LM                  = 3000,
-  KERN_TYPE_ORACLEH             = 3100,
-  KERN_TYPE_BCRYPT              = 3200,
-  KERN_TYPE_MD5_SLT_MD5_PW      = 3710,
-  KERN_TYPE_MD5_SLT_PW_SLT      = 3800,
-  KERN_TYPE_MD5U5               = 4300,
-  KERN_TYPE_MD5U5_PWSLT1        = 4310,
-  KERN_TYPE_MD5_SHA1            = 4400,
-  KERN_TYPE_SHA11               = 4500,
-  KERN_TYPE_SHA1_MD5            = 4700,
-  KERN_TYPE_MD5_CHAP            = 4800,
-  KERN_TYPE_SHA1_SLT_PW_SLT     = 4900,
-  KERN_TYPE_KECCAK              = 5000,
-  KERN_TYPE_MD5H                = 5100,
-  KERN_TYPE_PSAFE3              = 5200,
-  KERN_TYPE_IKEPSK_MD5          = 5300,
-  KERN_TYPE_IKEPSK_SHA1         = 5400,
-  KERN_TYPE_NETNTLMv1           = 5500,
-  KERN_TYPE_NETNTLMv2           = 5600,
-  KERN_TYPE_ANDROIDPIN          = 5800,
-  KERN_TYPE_RIPEMD160           = 6000,
-  KERN_TYPE_WHIRLPOOL           = 6100,
-  KERN_TYPE_TCRIPEMD160_XTS512  = 6211,
-  KERN_TYPE_TCRIPEMD160_XTS1024 = 6212,
-  KERN_TYPE_TCRIPEMD160_XTS1536 = 6213,
-  KERN_TYPE_TCSHA512_XTS512     = 6221,
-  KERN_TYPE_TCSHA512_XTS1024    = 6222,
-  KERN_TYPE_TCSHA512_XTS1536    = 6223,
-  KERN_TYPE_TCWHIRLPOOL_XTS512  = 6231,
-  KERN_TYPE_TCWHIRLPOOL_XTS1024 = 6232,
-  KERN_TYPE_TCWHIRLPOOL_XTS1536 = 6233,
-  KERN_TYPE_VCSHA256_XTS512     = 13751,
-  KERN_TYPE_VCSHA256_XTS1024    = 13752,
-  KERN_TYPE_VCSHA256_XTS1536    = 13753,
-  KERN_TYPE_MD5AIX              = 6300,
-  KERN_TYPE_SHA256AIX           = 6400,
-  KERN_TYPE_SHA512AIX           = 6500,
-  KERN_TYPE_AGILEKEY            = 6600,
-  KERN_TYPE_SHA1AIX             = 6700,
-  KERN_TYPE_LASTPASS            = 6800,
-  KERN_TYPE_GOST                = 6900,
-  KERN_TYPE_PBKDF2_SHA512       = 7100,
-  KERN_TYPE_RAKP                = 7300,
-  KERN_TYPE_SHA256CRYPT         = 7400,
-  KERN_TYPE_KRB5PA              = 7500,
-  KERN_TYPE_SHA1_SLT_SHA1_PW    = 7600,
-  KERN_TYPE_SAPB                = 7700,
-  KERN_TYPE_SAPG                = 7800,
-  KERN_TYPE_DRUPAL7             = 7900,
-  KERN_TYPE_SYBASEASE           = 8000,
-  KERN_TYPE_NETSCALER           = 8100,
-  KERN_TYPE_CLOUDKEY            = 8200,
-  KERN_TYPE_NSEC3               = 8300,
-  KERN_TYPE_WBB3                = 8400,
-  KERN_TYPE_RACF                = 8500,
-  KERN_TYPE_LOTUS5              = 8600,
-  KERN_TYPE_LOTUS6              = 8700,
-  KERN_TYPE_ANDROIDFDE          = 8800,
-  KERN_TYPE_SCRYPT              = 8900,
-  KERN_TYPE_PSAFE2              = 9000,
-  KERN_TYPE_LOTUS8              = 9100,
-  KERN_TYPE_OFFICE2007          = 9400,
-  KERN_TYPE_OFFICE2010          = 9500,
-  KERN_TYPE_OFFICE2013          = 9600,
-  KERN_TYPE_OLDOFFICE01         = 9700,
-  KERN_TYPE_OLDOFFICE01CM1      = 9710,
-  KERN_TYPE_OLDOFFICE01CM2      = 9720,
-  KERN_TYPE_OLDOFFICE34         = 9800,
-  KERN_TYPE_OLDOFFICE34CM1      = 9810,
-  KERN_TYPE_OLDOFFICE34CM2      = 9820,
-  KERN_TYPE_RADMIN2             = 9900,
-  KERN_TYPE_SIPHASH             = 10100,
-  KERN_TYPE_SAPH_SHA1           = 10300,
-  KERN_TYPE_PDF11               = 10400,
-  KERN_TYPE_PDF11CM1            = 10410,
-  KERN_TYPE_PDF11CM2            = 10420,
-  KERN_TYPE_PDF14               = 10500,
-  KERN_TYPE_PDF17L8             = 10700,
-  KERN_TYPE_SHA384              = 10800,
-  KERN_TYPE_PBKDF2_SHA256       = 10900,
-  KERN_TYPE_PRESTASHOP          = 11000,
-  KERN_TYPE_POSTGRESQL_AUTH     = 11100,
-  KERN_TYPE_MYSQL_AUTH          = 11200,
-  KERN_TYPE_BITCOIN_WALLET      = 11300,
-  KERN_TYPE_SIP_AUTH            = 11400,
-  KERN_TYPE_CRC32               = 11500,
-  KERN_TYPE_SEVEN_ZIP           = 11600,
-  KERN_TYPE_GOST_2012SBOG_256   = 11700,
-  KERN_TYPE_GOST_2012SBOG_512   = 11800,
-  KERN_TYPE_PBKDF2_MD5          = 11900,
-  KERN_TYPE_PBKDF2_SHA1         = 12000,
-  KERN_TYPE_ECRYPTFS            = 12200,
-  KERN_TYPE_ORACLET             = 12300,
-  KERN_TYPE_BSDICRYPT           = 12400,
-  KERN_TYPE_RAR3                = 12500,
-  KERN_TYPE_CF10                = 12600,
-  KERN_TYPE_MYWALLET            = 12700,
-  KERN_TYPE_MS_DRSR             = 12800,
-  KERN_TYPE_ANDROIDFDE_SAMSUNG  = 12900,
-  KERN_TYPE_RAR5                = 13000,
-  KERN_TYPE_KRB5TGS             = 13100,
-  KERN_TYPE_AXCRYPT             = 13200,
-  KERN_TYPE_SHA1_AXCRYPT        = 13300,
-  KERN_TYPE_KEEPASS             = 13400,
-  KERN_TYPE_PSTOKEN             = 13500,
-  KERN_TYPE_ZIP2                = 13600,
-  KERN_TYPE_WIN8PHONE           = 13800,
-  KERN_TYPE_OPENCART            = 13900,
-  KERN_TYPE_DES                 = 14000,
-  KERN_TYPE_3DES                = 14100,
-  KERN_TYPE_SHA1CX              = 14400,
-  KERN_TYPE_PLAINTEXT           = 99999,
+  KERN_TYPE_MD5                     = 0,
+  KERN_TYPE_MD5_PWSLT               = 10,
+  KERN_TYPE_MD5_SLTPW               = 20,
+  KERN_TYPE_MD5_PWUSLT              = 30,
+  KERN_TYPE_MD5_SLTPWU              = 40,
+  KERN_TYPE_HMACMD5_PW              = 50,
+  KERN_TYPE_HMACMD5_SLT             = 60,
+  KERN_TYPE_SHA1                    = 100,
+  KERN_TYPE_SHA1_PWSLT              = 110,
+  KERN_TYPE_SHA1_SLTPW              = 120,
+  KERN_TYPE_SHA1_PWUSLT             = 130,
+  KERN_TYPE_SHA1_SLTPWU             = 140,
+  KERN_TYPE_HMACSHA1_PW             = 150,
+  KERN_TYPE_HMACSHA1_SLT            = 160,
+  KERN_TYPE_MYSQL                   = 200,
+  KERN_TYPE_MYSQL41                 = 300,
+  KERN_TYPE_PHPASS                  = 400,
+  KERN_TYPE_MD5CRYPT                = 500,
+  KERN_TYPE_BLAKE2B                 = 600,
+  KERN_TYPE_MD4                     = 900,
+  KERN_TYPE_MD4_PWU                 = 1000,
+  KERN_TYPE_MD44_PWUSLT             = 1100,
+  KERN_TYPE_SHA224                  = 1300,
+  KERN_TYPE_SHA256                  = 1400,
+  KERN_TYPE_SHA256_PWSLT            = 1410,
+  KERN_TYPE_SHA256_SLTPW            = 1420,
+  KERN_TYPE_SHA256_PWUSLT           = 1430,
+  KERN_TYPE_SHA256_SLTPWU           = 1440,
+  KERN_TYPE_HMACSHA256_PW           = 1450,
+  KERN_TYPE_HMACSHA256_SLT          = 1460,
+  KERN_TYPE_DESCRYPT                = 1500,
+  KERN_TYPE_APR1CRYPT               = 1600,
+  KERN_TYPE_SHA512                  = 1700,
+  KERN_TYPE_SHA512_PWSLT            = 1710,
+  KERN_TYPE_SHA512_SLTPW            = 1720,
+  KERN_TYPE_SHA512_PWSLTU           = 1730,
+  KERN_TYPE_SHA512_SLTPWU           = 1740,
+  KERN_TYPE_HMACSHA512_PW           = 1750,
+  KERN_TYPE_HMACSHA512_SLT          = 1760,
+  KERN_TYPE_SHA512CRYPT             = 1800,
+  KERN_TYPE_STDOUT                  = 2000,
+  KERN_TYPE_DCC2                    = 2100,
+  KERN_TYPE_MD5PIX                  = 2400,
+  KERN_TYPE_MD5ASA                  = 2410,
+  KERN_TYPE_WPA                     = 2500,
+  KERN_TYPE_MD55                    = 2600,
+  KERN_TYPE_MD55_PWSLT1             = 2610,
+  KERN_TYPE_MD55_PWSLT2             = 2710,
+  KERN_TYPE_MD55_SLTPW              = 2810,
+  KERN_TYPE_LM                      = 3000,
+  KERN_TYPE_ORACLEH                 = 3100,
+  KERN_TYPE_BCRYPT                  = 3200,
+  KERN_TYPE_MD5_SLT_MD5_PW          = 3710,
+  KERN_TYPE_MD5_SLT_PW_SLT          = 3800,
+  KERN_TYPE_MD55_PWSLT              = 3910,
+  KERN_TYPE_MD5_SLT_MD5_SLT_PW      = 4010,
+  KERN_TYPE_MD5_SLT_MD5_PW_SLT      = 4110,
+  KERN_TYPE_MD5U5                   = 4300,
+  KERN_TYPE_MD5U5_PWSLT1            = 4310,
+  KERN_TYPE_MD5_SHA1                = 4400,
+  KERN_TYPE_SHA11                   = 4500,
+  KERN_TYPE_SHA1_SLT_SHA1_PW        = 4520,
+  KERN_TYPE_SHA1_MD5                = 4700,
+  KERN_TYPE_MD5_CHAP                = 4800,
+  KERN_TYPE_SHA1_SLT_PW_SLT         = 4900,
+  KERN_TYPE_KECCAK                  = 5000,
+  KERN_TYPE_MD5H                    = 5100,
+  KERN_TYPE_PSAFE3                  = 5200,
+  KERN_TYPE_IKEPSK_MD5              = 5300,
+  KERN_TYPE_IKEPSK_SHA1             = 5400,
+  KERN_TYPE_NETNTLMv1               = 5500,
+  KERN_TYPE_NETNTLMv2               = 5600,
+  KERN_TYPE_ANDROIDPIN              = 5800,
+  KERN_TYPE_RIPEMD160               = 6000,
+  KERN_TYPE_WHIRLPOOL               = 6100,
+  KERN_TYPE_TCRIPEMD160_XTS512      = 6211,
+  KERN_TYPE_TCRIPEMD160_XTS1024     = 6212,
+  KERN_TYPE_TCRIPEMD160_XTS1536     = 6213,
+  KERN_TYPE_TCSHA512_XTS512         = 6221,
+  KERN_TYPE_TCSHA512_XTS1024        = 6222,
+  KERN_TYPE_TCSHA512_XTS1536        = 6223,
+  KERN_TYPE_TCWHIRLPOOL_XTS512      = 6231,
+  KERN_TYPE_TCWHIRLPOOL_XTS1024     = 6232,
+  KERN_TYPE_TCWHIRLPOOL_XTS1536     = 6233,
+  KERN_TYPE_VCSHA256_XTS512         = 13751,
+  KERN_TYPE_VCSHA256_XTS1024        = 13752,
+  KERN_TYPE_VCSHA256_XTS1536        = 13753,
+  KERN_TYPE_MD5AIX                  = 6300,
+  KERN_TYPE_SHA256AIX               = 6400,
+  KERN_TYPE_SHA512AIX               = 6500,
+  KERN_TYPE_AGILEKEY                = 6600,
+  KERN_TYPE_SHA1AIX                 = 6700,
+  KERN_TYPE_LASTPASS                = 6800,
+  KERN_TYPE_GOST                    = 6900,
+  KERN_TYPE_FORTIGATE               = 7000,
+  KERN_TYPE_PBKDF2_SHA512           = 7100,
+  KERN_TYPE_RAKP                    = 7300,
+  KERN_TYPE_SHA256CRYPT             = 7400,
+  KERN_TYPE_KRB5PA                  = 7500,
+  KERN_TYPE_SAPB                    = 7700,
+  KERN_TYPE_SAPG                    = 7800,
+  KERN_TYPE_DRUPAL7                 = 7900,
+  KERN_TYPE_SYBASEASE               = 8000,
+  KERN_TYPE_NETSCALER               = 8100,
+  KERN_TYPE_CLOUDKEY                = 8200,
+  KERN_TYPE_NSEC3                   = 8300,
+  KERN_TYPE_WBB3                    = 8400,
+  KERN_TYPE_RACF                    = 8500,
+  KERN_TYPE_LOTUS5                  = 8600,
+  KERN_TYPE_LOTUS6                  = 8700,
+  KERN_TYPE_ANDROIDFDE              = 8800,
+  KERN_TYPE_SCRYPT                  = 8900,
+  KERN_TYPE_PSAFE2                  = 9000,
+  KERN_TYPE_LOTUS8                  = 9100,
+  KERN_TYPE_OFFICE2007              = 9400,
+  KERN_TYPE_OFFICE2010              = 9500,
+  KERN_TYPE_OFFICE2013              = 9600,
+  KERN_TYPE_OLDOFFICE01             = 9700,
+  KERN_TYPE_OLDOFFICE01CM1          = 9710,
+  KERN_TYPE_OLDOFFICE01CM2          = 9720,
+  KERN_TYPE_OLDOFFICE34             = 9800,
+  KERN_TYPE_OLDOFFICE34CM1          = 9810,
+  KERN_TYPE_OLDOFFICE34CM2          = 9820,
+  KERN_TYPE_RADMIN2                 = 9900,
+  KERN_TYPE_SIPHASH                 = 10100,
+  KERN_TYPE_SAPH_SHA1               = 10300,
+  KERN_TYPE_PDF11                   = 10400,
+  KERN_TYPE_PDF11CM1                = 10410,
+  KERN_TYPE_PDF11CM2                = 10420,
+  KERN_TYPE_PDF14                   = 10500,
+  KERN_TYPE_PDF17L8                 = 10700,
+  KERN_TYPE_SHA384                  = 10800,
+  KERN_TYPE_PBKDF2_SHA256           = 10900,
+  KERN_TYPE_PRESTASHOP              = 11000,
+  KERN_TYPE_POSTGRESQL_AUTH         = 11100,
+  KERN_TYPE_MYSQL_AUTH              = 11200,
+  KERN_TYPE_BITCOIN_WALLET          = 11300,
+  KERN_TYPE_SIP_AUTH                = 11400,
+  KERN_TYPE_CRC32                   = 11500,
+  KERN_TYPE_SEVEN_ZIP               = 11600,
+  KERN_TYPE_GOST_2012SBOG_256       = 11700,
+  KERN_TYPE_GOST_2012SBOG_512       = 11800,
+  KERN_TYPE_PBKDF2_MD5              = 11900,
+  KERN_TYPE_PBKDF2_SHA1             = 12000,
+  KERN_TYPE_ECRYPTFS                = 12200,
+  KERN_TYPE_ORACLET                 = 12300,
+  KERN_TYPE_BSDICRYPT               = 12400,
+  KERN_TYPE_RAR3                    = 12500,
+  KERN_TYPE_CF10                    = 12600,
+  KERN_TYPE_MYWALLET                = 12700,
+  KERN_TYPE_MS_DRSR                 = 12800,
+  KERN_TYPE_ANDROIDFDE_SAMSUNG      = 12900,
+  KERN_TYPE_RAR5                    = 13000,
+  KERN_TYPE_KRB5TGS                 = 13100,
+  KERN_TYPE_AXCRYPT                 = 13200,
+  KERN_TYPE_SHA1_AXCRYPT            = 13300,
+  KERN_TYPE_KEEPASS                 = 13400,
+  KERN_TYPE_PSTOKEN                 = 13500,
+  KERN_TYPE_ZIP2                    = 13600,
+  KERN_TYPE_WIN8PHONE               = 13800,
+  KERN_TYPE_OPENCART                = 13900,
+  KERN_TYPE_DES                     = 14000,
+  KERN_TYPE_3DES                    = 14100,
+  KERN_TYPE_SHA1CX                  = 14400,
+  KERN_TYPE_LUKS_SHA1_AES           = 14611,
+  KERN_TYPE_LUKS_SHA1_SERPENT       = 14612,
+  KERN_TYPE_LUKS_SHA1_TWOFISH       = 14613,
+  KERN_TYPE_LUKS_SHA256_AES         = 14621,
+  KERN_TYPE_LUKS_SHA256_SERPENT     = 14622,
+  KERN_TYPE_LUKS_SHA256_TWOFISH     = 14623,
+  KERN_TYPE_LUKS_SHA512_AES         = 14631,
+  KERN_TYPE_LUKS_SHA512_SERPENT     = 14632,
+  KERN_TYPE_LUKS_SHA512_TWOFISH     = 14633,
+  KERN_TYPE_LUKS_RIPEMD160_AES      = 14641,
+  KERN_TYPE_LUKS_RIPEMD160_SERPENT  = 14642,
+  KERN_TYPE_LUKS_RIPEMD160_TWOFISH  = 14643,
+  KERN_TYPE_LUKS_WHIRLPOOL_AES      = 14651,
+  KERN_TYPE_LUKS_WHIRLPOOL_SERPENT  = 14652,
+  KERN_TYPE_LUKS_WHIRLPOOL_TWOFISH  = 14653,
+  KERN_TYPE_ITUNES_BACKUP_9         = 14700,
+  KERN_TYPE_ITUNES_BACKUP_10        = 14800,
+  KERN_TYPE_SKIP32                  = 14900,
+  KERN_TYPE_FILEZILLA_SERVER        = 15000,
+  KERN_TYPE_NETBSD_SHA1CRYPT        = 15100,
+  KERN_TYPE_DPAPIMK                 = 15300,
+  KERN_TYPE_CHACHA20                = 15400,
+  KERN_TYPE_JKS_SHA1                = 15500,
+  KERN_TYPE_ETHEREUM_PBKDF2         = 15600,
+  KERN_TYPE_ETHEREUM_SCRYPT         = 15700,
+  KERN_TYPE_PLAINTEXT               = 99999,
 
 } kern_type_t;
 
@@ -1338,12 +1672,21 @@ typedef enum rounds_count
    ROUNDS_BSDICRYPT          = 2900,
    ROUNDS_RAR3               = 262144,
    ROUNDS_MYWALLET           = 10,
+   ROUNDS_MYWALLETV2         = 5000,
    ROUNDS_MS_DRSR            = 100,
    ROUNDS_ANDROIDFDE_SAMSUNG = 4096,
    ROUNDS_RAR5               = (1 << 15),
    ROUNDS_AXCRYPT            = 10000,
    ROUNDS_KEEPASS            = 6000,
    ROUNDS_ZIP2               = 1000,
+   ROUNDS_LUKS               = 163044, // this equal to jtr -test
+   ROUNDS_ITUNES9_BACKUP     = 10000,
+   ROUNDS_ITUNES101_BACKUP   = 10000000, // wtf, i mean, really?
+   ROUNDS_ITUNES102_BACKUP   = 10000,
+   ROUNDS_ATLASSIAN          = 10000,
+   ROUNDS_NETBSD_SHA1CRYPT   = 20000,
+   ROUNDS_DPAPIMK            = 24000 - 1, // from 4000 to 24000 (possibly more)
+   ROUNDS_ETHEREUM_PBKDF2    = 262144 - 1,
    ROUNDS_STDOUT             = 0
 
 } rounds_count_t;
@@ -1364,6 +1707,8 @@ int joomla_parse_hash             (u8 *input_buf, u32 input_len, hash_t *hash_bu
 int postgresql_parse_hash         (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
 int netscreen_parse_hash          (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
 int keccak_parse_hash             (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
+int blake2b_parse_hash            (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
+int chacha20_parse_hash           (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
 int lm_parse_hash                 (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
 int md4_parse_hash                (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
 int md5_parse_hash                (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
@@ -1390,6 +1735,7 @@ int sha1_parse_hash               (u8 *input_buf, u32 input_len, hash_t *hash_bu
 int sha1b64_parse_hash            (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
 int sha1b64s_parse_hash           (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
 int sha1s_parse_hash              (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
+int sha1sha1_parse_hash           (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
 int sha224_parse_hash             (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
 int sha256_parse_hash             (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
 int sha256s_parse_hash            (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
@@ -1470,6 +1816,7 @@ int siphash_parse_hash            (u8 *input_buf, u32 input_len, hash_t *hash_bu
 int crammd5_parse_hash            (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
 int saph_sha1_parse_hash          (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
 int redmine_parse_hash            (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
+int punbb_parse_hash              (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
 int pdf11_parse_hash              (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
 int pdf11cm1_parse_hash           (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
 int pdf11cm2_parse_hash           (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
@@ -1495,6 +1842,7 @@ int rar3hp_parse_hash             (u8 *input_buf, u32 input_len, hash_t *hash_bu
 int rar5_parse_hash               (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
 int cf10_parse_hash               (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
 int mywallet_parse_hash           (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
+int mywalletv2_parse_hash         (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
 int ms_drsr_parse_hash            (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
 int androidfde_samsung_parse_hash (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
 int axcrypt_parse_hash            (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
@@ -1510,6 +1858,24 @@ int win8phone_parse_hash          (u8 *input_buf, u32 input_len, hash_t *hash_bu
 int opencart_parse_hash           (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
 int plaintext_parse_hash          (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
 int sha1cx_parse_hash             (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
+int luks_parse_hash               (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED       hashconfig_t *hashconfig, const int keyslot_idx);
+int itunes_backup_parse_hash      (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
+int skip32_parse_hash             (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
+int fortigate_parse_hash          (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
+int sha256b64s_parse_hash         (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
+int filezilla_server_parse_hash   (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
+int netbsd_sha1crypt_parse_hash   (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
+int atlassian_parse_hash          (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
+int dpapimk_parse_hash            (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
+int jks_sha1_parse_hash           (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
+int ethereum_pbkdf2_parse_hash    (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
+int ethereum_scrypt_parse_hash    (u8 *input_buf, u32 input_len, hash_t *hash_buf, MAYBE_UNUSED const hashconfig_t *hashconfig);
+
+/**
+ * hook functions
+ */
+
+void seven_zip_hook_func (hc_device_param_t *device_param, hashes_t *hashes, const u32 salt_pos, const u32 pws_cnt);
 
 /**
  * output functions
@@ -1519,18 +1885,17 @@ char *stroptitype (const u32 opti_type);
 char *strhashtype (const u32 hash_mode);
 char *strparser   (const u32 parser_status);
 
-void to_hccap_t (hashcat_ctx_t *hashcat_ctx, hccap_t *hccap, const u32 salt_pos, const u32 digest_pos);
-
-void wpa_essid_reuse (hashcat_ctx_t *hashcat_ctx);
+int check_old_hccap (const char *hashfile);
+void to_hccapx_t (hashcat_ctx_t *hashcat_ctx, hccapx_t *hccapx, const u32 salt_pos, const u32 digest_pos);
 
 int ascii_digest (hashcat_ctx_t *hashcat_ctx, char *out_buf, const size_t out_len, const u32 salt_pos, const u32 digest_pos);
 
-int     hashconfig_init               (hashcat_ctx_t *hashcat_ctx);
-void    hashconfig_destroy            (hashcat_ctx_t *hashcat_ctx);
-u32     hashconfig_get_kernel_threads (hashcat_ctx_t *hashcat_ctx, const hc_device_param_t *device_param);
-u32     hashconfig_get_kernel_loops   (hashcat_ctx_t *hashcat_ctx);
-int     hashconfig_general_defaults   (hashcat_ctx_t *hashcat_ctx);
-void    hashconfig_benchmark_defaults (hashcat_ctx_t *hashcat_ctx, salt_t *salt, void *esalt);
-char   *hashconfig_benchmark_mask     (hashcat_ctx_t *hashcat_ctx);
+int         hashconfig_init               (hashcat_ctx_t *hashcat_ctx);
+void        hashconfig_destroy            (hashcat_ctx_t *hashcat_ctx);
+u32         hashconfig_get_kernel_threads (hashcat_ctx_t *hashcat_ctx, const hc_device_param_t *device_param);
+u32         hashconfig_get_kernel_loops   (hashcat_ctx_t *hashcat_ctx);
+int         hashconfig_general_defaults   (hashcat_ctx_t *hashcat_ctx);
+void        hashconfig_benchmark_defaults (hashcat_ctx_t *hashcat_ctx, salt_t *salt, void *esalt, void *hook_salt);
+const char *hashconfig_benchmark_mask     (hashcat_ctx_t *hashcat_ctx);
 
 #endif // _INTERFACE_H
