@@ -63,19 +63,13 @@ int sort_by_salt (const void *v1, const void *v2)
 
   if (res2 != 0) return (res2);
 
-  u32 n;
-
-  n = 16;
-
-  while (n--)
+  for (int n = 0; n < 16; n++)
   {
     if (s1->salt_buf[n] > s2->salt_buf[n]) return  1;
     if (s1->salt_buf[n] < s2->salt_buf[n]) return -1;
   }
 
-  n = 8;
-
-  while (n--)
+  for (int n = 0; n < 8; n++)
   {
     if (s1->salt_buf_pc[n] > s2->salt_buf_pc[n]) return  1;
     if (s1->salt_buf_pc[n] < s2->salt_buf_pc[n]) return -1;
@@ -124,7 +118,7 @@ int save_hash (hashcat_ctx_t *hashcat_ctx)
   hashconfig_t    *hashconfig   = hashcat_ctx->hashconfig;
   user_options_t  *user_options = hashcat_ctx->user_options;
 
-  char *hashfile = hashes->hashfile;
+  const char *hashfile = hashes->hashfile;
 
   char new_hashfile[256] = { 0 };
   char old_hashfile[256] = { 0 };
@@ -140,7 +134,7 @@ int save_hash (hashcat_ctx_t *hashcat_ctx)
 
   if (fp == NULL)
   {
-    event_log_error (hashcat_ctx, "%s: %m", new_hashfile);
+    event_log_error (hashcat_ctx, "%s: %s", new_hashfile, strerror (errno));
 
     return -1;
   }
@@ -149,7 +143,7 @@ int save_hash (hashcat_ctx_t *hashcat_ctx)
   {
     fclose (fp);
 
-    event_log_error (hashcat_ctx, "%s: %m", new_hashfile);
+    event_log_error (hashcat_ctx, "%s: %s", new_hashfile, strerror (errno));
 
     return -1;
   }
@@ -172,11 +166,11 @@ int save_hash (hashcat_ctx_t *hashcat_ctx)
       {
         if (hashconfig->hash_mode == 2500)
         {
-          hccap_t hccap;
+          hccapx_t hccapx;
 
-          to_hccap_t (hashcat_ctx, &hccap, salt_pos, digest_pos);
+          to_hccapx_t (hashcat_ctx, &hccapx, salt_pos, digest_pos);
 
-          fwrite (&hccap, sizeof (hccap_t), 1, fp);
+          fwrite (&hccapx, sizeof (hccapx_t), 1, fp);
         }
         else
         {
@@ -215,7 +209,7 @@ int save_hash (hashcat_ctx_t *hashcat_ctx)
 
   if (rename (hashfile, old_hashfile) != 0)
   {
-    event_log_error (hashcat_ctx, "Rename file '%s' to '%s': %m", hashfile, old_hashfile);
+    event_log_error (hashcat_ctx, "Rename file '%s' to '%s': %s", hashfile, old_hashfile, strerror (errno));
 
     return -1;
   }
@@ -224,7 +218,7 @@ int save_hash (hashcat_ctx_t *hashcat_ctx)
 
   if (rename (new_hashfile, hashfile) != 0)
   {
-    event_log_error (hashcat_ctx, "Rename file '%s' to '%s': %m", new_hashfile, hashfile);
+    event_log_error (hashcat_ctx, "Rename file '%s' to '%s': %s", new_hashfile, hashfile, strerror (errno));
 
     return -1;
   }
@@ -457,11 +451,9 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
 
   u32 hashes_avail = 0;
 
-  if ((user_options->benchmark == false) && (user_options->stdout_flag == false))
+  if ((user_options->benchmark == false) && (user_options->stdout_flag == false) && (user_options->keyspace == false))
   {
-    hc_stat_t f;
-
-    hashlist_mode = (hc_stat (hash_or_file, &f) == 0) ? HL_MODE_FILE : HL_MODE_ARG;
+    hashlist_mode = (hc_path_exist (hash_or_file) == true) ? HL_MODE_FILE : HL_MODE_ARG;
 
     if (hashconfig->opts_type & OPTS_TYPE_BINARY_HASHFILE)
     {
@@ -480,12 +472,39 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
 
         if (hc_stat (hashes->hashfile, &st) == -1)
         {
-          event_log_error (hashcat_ctx, "%s: %m", hashes->hashfile);
+          event_log_error (hashcat_ctx, "%s: %s", hashes->hashfile, strerror (errno));
 
           return -1;
         }
 
-        hashes_avail = st.st_size / sizeof (hccap_t);
+        // 392 = old hccap_t size
+
+        if ((st.st_size % 392) == 0)
+        {
+          const int rc = check_old_hccap (hashes->hashfile);
+
+          if (rc == 1)
+          {
+            event_log_error (hashcat_ctx, "%s: Old hccap format detected! You need to update: https://hashcat.net/q/hccapx", hashes->hashfile);
+
+            return -1;
+          }
+        }
+
+        hashes_avail = st.st_size / sizeof (hccapx_t);
+      }
+      else if (hashconfig->hash_mode == 14600)
+      {
+        hc_stat_t st;
+
+        if (hc_stat (hashes->hashfile, &st) == -1)
+        {
+          event_log_error (hashcat_ctx, "%s: %s", hashes->hashfile, strerror (errno));
+
+          return -1;
+        }
+
+        hashes_avail = LUKS_NUMKEYS;
       }
       else
       {
@@ -502,7 +521,7 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
 
       if ((fp = fopen (hashfile, "rb")) == NULL)
       {
-        event_log_error (hashcat_ctx, "%s: %m", hashfile);
+        event_log_error (hashcat_ctx, "%s: %s", hashfile, strerror (errno));
 
         return -1;
       }
@@ -517,7 +536,7 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
 
       if (hashes_avail == 0)
       {
-        event_log_error (hashcat_ctx, "hashfile is empty or corrupt");
+        event_log_error (hashcat_ctx, "hashfile is empty or corrupt.");
 
         fclose (fp);
 
@@ -526,9 +545,9 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
 
       hashlist_format = hlfmt_detect (hashcat_ctx, fp, 100); // 100 = max numbers to "scan". could be hashes_avail, too
 
-      if ((user_options->remove == 1) && (hashlist_format != HLFMT_HASHCAT))
+      if ((user_options->remove == true) && (hashlist_format != HLFMT_HASHCAT))
       {
-        event_log_error (hashcat_ctx, "remove not supported in native hashfile-format mode");
+        event_log_error (hashcat_ctx, "Use of --remove is not supported in native hashfile-format mode.");
 
         fclose (fp);
 
@@ -554,16 +573,13 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
    * load hashes, part II: allocate required memory, set pointers
    */
 
-  hash_t *hashes_buf  = NULL;
-  void   *digests_buf = NULL;
-  salt_t *salts_buf   = NULL;
-  void   *esalts_buf  = NULL;
+  hash_t *hashes_buf     = (hash_t *) hccalloc (hashes_avail, sizeof (hash_t));
+  void   *digests_buf    = (void *)   hccalloc (hashes_avail, hashconfig->dgst_size);
+  salt_t *salts_buf      = NULL;
+  void   *esalts_buf     = NULL;
+  void   *hook_salts_buf = NULL;
 
-  hashes_buf = (hash_t *) hccalloc (hashes_avail, sizeof (hash_t));
-
-  digests_buf = (void *) hccalloc (hashes_avail, hashconfig->dgst_size);
-
-  if ((user_options->username == true) || (hashconfig->opts_type & OPTS_TYPE_HASH_COPY))
+  if ((user_options->username == true) || (hashconfig->opts_type & OPTS_TYPE_HASH_COPY) || (hashconfig->opts_type & OPTS_TYPE_HASH_SPLIT))
   {
     u32 hash_pos;
 
@@ -575,12 +591,20 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
 
       if (user_options->username == true)
       {
-        hash_info->user = (user_t*) hcmalloc (sizeof (user_t));
+        hash_info->user = (user_t *) hcmalloc (sizeof (user_t));
       }
 
-      if (user_options->benchmark == true)
+      if (hashconfig->opts_type & OPTS_TYPE_HASH_COPY)
       {
-        hash_info->orighash = (char *) hcmalloc (256);
+        if (user_options->benchmark == false)
+        {
+          hash_info->orighash = (char *) hcmalloc (256);
+        }
+      }
+
+      if (hashconfig->opts_type & OPTS_TYPE_HASH_SPLIT)
+      {
+        hash_info->split = (split_t *) hcmalloc (sizeof (split_t));
       }
     }
   }
@@ -593,6 +617,11 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
     {
       esalts_buf = (void *) hccalloc (hashes_avail, hashconfig->esalt_size);
     }
+
+    if (hashconfig->hook_salt_size)
+    {
+      hook_salts_buf = (seven_zip_hook_salt_t *) hccalloc (hashes_avail, hashconfig->hook_salt_size);
+    }
   }
   else
   {
@@ -611,6 +640,11 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
       {
         hashes_buf[hash_pos].esalt = ((char *) esalts_buf) + (hash_pos * hashconfig->esalt_size);
       }
+
+      if (hashconfig->hook_salt_size)
+      {
+        hashes_buf[hash_pos].hook_salt = ((char *) hook_salts_buf) + (hash_pos * hashconfig->hook_salt_size);
+      }
     }
     else
     {
@@ -618,10 +652,11 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
     }
   }
 
-  hashes->hashes_buf  = hashes_buf;
-  hashes->digests_buf = digests_buf;
-  hashes->salts_buf   = salts_buf;
-  hashes->esalts_buf  = esalts_buf;
+  hashes->hashes_buf     = hashes_buf;
+  hashes->digests_buf    = digests_buf;
+  hashes->salts_buf      = salts_buf;
+  hashes->esalts_buf     = esalts_buf;
+  hashes->hook_salts_buf = hook_salts_buf;
 
   /**
    * load hashes, part III: parse hashes or generate them if benchmark
@@ -631,7 +666,7 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
 
   if (user_options->benchmark == true)
   {
-    hashconfig_benchmark_defaults (hashcat_ctx, hashes_buf[0].salt, hashes_buf[0].esalt);
+    hashconfig_benchmark_defaults (hashcat_ctx, hashes_buf[0].salt, hashes_buf[0].esalt, hashes_buf[0].hook_salt);
 
     hashes->hashfile = "-";
 
@@ -670,7 +705,7 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
 
       if (hash_fmt_error)
       {
-        event_log_warning (hashcat_ctx, "Failed to parse hashes using the '%s' format", strhlfmt (hashlist_format));
+        event_log_warning (hashcat_ctx, "Failed to parse hashes using the '%s' format.", strhlfmt (hashlist_format));
       }
       else
       {
@@ -686,17 +721,20 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
           memset (hashes_buf[0].salt, 0, sizeof (salt_t));
         }
 
+        if (hashconfig->esalt_size)
+        {
+          memset (hashes_buf[0].esalt, 0, hashconfig->esalt_size);
+        }
+
+        if (hashconfig->hook_salt_size)
+        {
+          memset (hashes_buf[0].hook_salt, 0, hashconfig->hook_salt_size);
+        }
+
         int parser_status = PARSER_OK;
 
         if (hashconfig->hash_mode == 2500)
         {
-          if (hash_len == 0)
-          {
-            event_log_error (hashcat_ctx, "hccap file not specified");
-
-            return -1;
-          }
-
           hashlist_mode = HL_MODE_FILE;
 
           hashes->hashlist_mode = hashlist_mode;
@@ -705,36 +743,55 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
 
           if (fp == NULL)
           {
-            event_log_error (hashcat_ctx, "%s: %m", hash_buf);
+            event_log_error (hashcat_ctx, "%s: %s", hash_buf, strerror (errno));
 
             return -1;
           }
 
-          if (hashes_avail < 1)
-          {
-            event_log_error (hashcat_ctx, "hccap file is empty or corrupt");
-
-            fclose (fp);
-
-            return -1;
-          }
-
-          char *in = (char *) hcmalloc (sizeof (hccap_t));
+          char *in = (char *) hcmalloc (sizeof (hccapx_t));
 
           while (!feof (fp))
           {
-            const int nread = fread (in, sizeof (hccap_t), 1, fp);
+            const size_t nread = fread (in, sizeof (hccapx_t), 1, fp);
 
             if (nread == 0) break;
 
             if (hashes_avail == hashes_cnt)
             {
-              event_log_warning (hashcat_ctx, "Hashfile '%s': File changed during runtime, skipping new data", hash_buf);
+              event_log_warning (hashcat_ctx, "Hashfile '%s': File changed during runtime. Skipping new data.", hash_buf);
 
               break;
             }
 
-            parser_status = hashconfig->parse_func ((u8 *) in, sizeof (hccap_t), &hashes_buf[hashes_cnt], hashconfig);
+            if (hashconfig->is_salted)
+            {
+              memset (hashes_buf[hashes_cnt].salt, 0, sizeof (salt_t));
+            }
+
+            if (hashconfig->esalt_size)
+            {
+              memset (hashes_buf[hashes_cnt].esalt, 0, hashconfig->esalt_size);
+
+              if (user_options->hash_mode == 2500)
+              {
+                wpa_t *wpa = (wpa_t *) hashes_buf[hashes_cnt].esalt;
+
+                if (user_options->hccapx_message_pair_chgd == true)
+                {
+                  wpa->message_pair_chgd = (int) user_options->hccapx_message_pair_chgd;
+                  wpa->message_pair      = (u8)  user_options->hccapx_message_pair;
+                }
+
+                wpa->nonce_error_corrections = user_options->nonce_error_corrections;
+              }
+            }
+
+            if (hashconfig->hook_salt_size)
+            {
+              memset (hashes_buf[hashes_cnt].hook_salt, 0, hashconfig->hook_salt_size);
+            }
+
+            parser_status = hashconfig->parse_func ((u8 *) in, sizeof (hccapx_t), &hashes_buf[hashes_cnt], hashconfig);
 
             if (parser_status != PARSER_OK)
             {
@@ -758,6 +815,9 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
 
             if (parser_status == PARSER_OK)
             {
+              hashes_buf[hashes_cnt].hash_info->split->split_group  = 0;
+              hashes_buf[hashes_cnt].hash_info->split->split_origin = SPLIT_ORIGIN_LEFT;
+
               hashes_cnt++;
             }
             else
@@ -769,6 +829,9 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
 
             if (parser_status == PARSER_OK)
             {
+              hashes_buf[hashes_cnt].hash_info->split->split_group  = 0;
+              hashes_buf[hashes_cnt].hash_info->split->split_origin = SPLIT_ORIGIN_RIGHT;
+
               hashes_cnt++;
             }
             else
@@ -782,6 +845,9 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
 
             if (parser_status == PARSER_OK)
             {
+              hashes_buf[hashes_cnt].hash_info->split->split_group  = 0;
+              hashes_buf[hashes_cnt].hash_info->split->split_origin = SPLIT_ORIGIN_NONE;
+
               hashes_cnt++;
             }
             else
@@ -790,6 +856,29 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
             }
           }
         }
+        else if (hashconfig->hash_mode == 14600)
+        {
+          hashlist_mode = HL_MODE_FILE;
+
+          hashes->hashlist_mode = hashlist_mode;
+
+          for (int keyslot_idx = 0; keyslot_idx < LUKS_NUMKEYS; keyslot_idx++)
+          {
+            parser_status = luks_parse_hash ((u8 *) hash_buf, hash_len, &hashes_buf[hashes_cnt], hashconfig, keyslot_idx);
+
+            if (parser_status != PARSER_OK)
+            {
+              if (parser_status != PARSER_LUKS_KEY_DISABLED)
+              {
+                event_log_warning (hashcat_ctx, "Hashfile '%s': %s", hash_buf, strparser (parser_status));
+              }
+
+              continue;
+            }
+
+            hashes_cnt++;
+          }
+        }
         else
         {
           parser_status = hashconfig->parse_func ((u8 *) hash_buf, hash_len, &hashes_buf[hashes_cnt], hashconfig);
@@ -807,13 +896,13 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
     }
     else if (hashlist_mode == HL_MODE_FILE)
     {
-      char *hashfile = hashes->hashfile;
+      const char *hashfile = hashes->hashfile;
 
       FILE *fp;
 
       if ((fp = fopen (hashfile, "rb")) == NULL)
       {
-        event_log_error (hashcat_ctx, "%s: %m", hashfile);
+        event_log_error (hashcat_ctx, "%s: %s", hashfile, strerror (errno));
 
         return -1;
       }
@@ -835,7 +924,7 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
 
         if (hashes_avail == hashes_cnt)
         {
-          event_log_warning (hashcat_ctx, "Hashfile '%s' on line %u: File changed during runtime, skipping new data", hashes->hashfile, line_num);
+          event_log_warning (hashcat_ctx, "Hashfile '%s' on line %u: File changed during runtime. Skipping new data.", hashes->hashfile, line_num);
 
           break;
         }
@@ -852,7 +941,7 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
 
         if (hash_fmt_error)
         {
-          event_log_warning (hashcat_ctx, "failed to parse hashes using the '%s' format", strhlfmt (hashlist_format));
+          event_log_warning (hashcat_ctx, "Failed to parse hashes using the '%s' format.", strhlfmt (hashlist_format));
 
           continue;
         }
@@ -864,22 +953,38 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
 
           hlfmt_user (hashcat_ctx, hashlist_format, line_buf, line_len, &user_buf, &user_len);
 
-          user_t **user = &hashes_buf[hashes_cnt].hash_info->user;
+          // special case:
+          // both hash_t need to have the username info if the pwdump format is used (i.e. we have 2 hashes for 3000, both with same user)
 
-          *user = (user_t *) hcmalloc (sizeof (user_t));
+          u32 hashes_per_user = 1;
 
-          user_t *user_ptr = *user;
-
-          if (user_buf != NULL)
+          if (hashconfig->hash_mode == 3000) // the following conditions should be true if (hashlist_format == HLFMT_PWDUMP)
           {
-            user_ptr->user_name = hcstrdup (user_buf);
-          }
-          else
-          {
-            user_ptr->user_name = hcstrdup ("");
+            if (hash_len == 32)
+            {
+              hashes_per_user = 2;
+            }
           }
 
-          user_ptr->user_len = user_len;
+          for (u32 i = 0; i < hashes_per_user; i++)
+          {
+            user_t **user = &hashes_buf[hashes_cnt + i].hash_info->user;
+
+            *user = (user_t *) hcmalloc (sizeof (user_t));
+
+            user_t *user_ptr = *user;
+
+            if (user_buf != NULL)
+            {
+              user_ptr->user_name = hcstrdup (user_buf);
+            }
+            else
+            {
+              user_ptr->user_name = hcstrdup ("");
+            }
+
+            user_ptr->user_len = user_len;
+          }
         }
 
         if (hashconfig->opts_type & OPTS_TYPE_HASH_COPY)
@@ -894,6 +999,16 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
           memset (hashes_buf[hashes_cnt].salt, 0, sizeof (salt_t));
         }
 
+        if (hashconfig->esalt_size)
+        {
+          memset (hashes_buf[hashes_cnt].esalt, 0, hashconfig->esalt_size);
+        }
+
+        if (hashconfig->hook_salt_size)
+        {
+          memset (hashes_buf[hashes_cnt].hook_salt, 0, hashconfig->hook_salt_size);
+        }
+
         if (hashconfig->hash_mode == 3000)
         {
           if (hash_len == 32)
@@ -907,6 +1022,9 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
               continue;
             }
 
+            hashes_buf[hashes_cnt].hash_info->split->split_group  = line_num;
+            hashes_buf[hashes_cnt].hash_info->split->split_origin = SPLIT_ORIGIN_LEFT;
+
             hashes_cnt++;
 
             parser_status = hashconfig->parse_func ((u8 *) hash_buf + 16, 16, &hashes_buf[hashes_cnt], hashconfig);
@@ -918,6 +1036,9 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
               continue;
             }
 
+            hashes_buf[hashes_cnt].hash_info->split->split_group  = line_num;
+            hashes_buf[hashes_cnt].hash_info->split->split_origin = SPLIT_ORIGIN_RIGHT;
+
             hashes_cnt++;
           }
           else
@@ -931,6 +1052,9 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
               continue;
             }
 
+            hashes_buf[hashes_cnt].hash_info->split->split_group  = line_num;
+            hashes_buf[hashes_cnt].hash_info->split->split_origin = SPLIT_ORIGIN_NONE;
+
             hashes_cnt++;
           }
         }
@@ -993,6 +1117,33 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
     EVENT (EVENT_HASHLIST_SORT_HASH_POST);
   }
 
+  if (hashconfig->hash_mode == 3000)
+  {
+    // update split split_neighbor after sorting
+    // see https://github.com/hashcat/hashcat/issues/1034 for good examples for testing
+
+    for (u32 i = 0; i < hashes_cnt; i++)
+    {
+      split_t *split1 = hashes_buf[i].hash_info->split;
+
+      if (split1->split_origin != SPLIT_ORIGIN_LEFT) continue;
+
+      for (u32 j = 0; j < hashes_cnt; j++)
+      {
+        split_t *split2 = hashes_buf[j].hash_info->split;
+
+        if (split2->split_origin != SPLIT_ORIGIN_RIGHT) continue;
+
+        if (split1->split_group != split2->split_group) continue;
+
+        split1->split_neighbor = j;
+        split2->split_neighbor = i;
+
+        break;
+      }
+    }
+  }
+
   return 0;
 }
 
@@ -1001,6 +1152,7 @@ int hashes_init_stage2 (hashcat_ctx_t *hashcat_ctx)
   hashconfig_t   *hashconfig   = hashcat_ctx->hashconfig;
   hashes_t       *hashes       = hashcat_ctx->hashes;
   user_options_t *user_options = hashcat_ctx->user_options;
+  potfile_ctx_t  *potfile_ctx  = hashcat_ctx->potfile_ctx;
 
   hash_t *hashes_buf = hashes->hashes_buf;
   u32     hashes_cnt = hashes->hashes_cnt;
@@ -1015,7 +1167,11 @@ int hashes_init_stage2 (hashcat_ctx_t *hashcat_ctx)
 
   for (u32 hashes_pos = 1; hashes_pos < hashes_cnt; hashes_pos++)
   {
-    if (hashconfig->is_salted)
+    if (potfile_ctx->keep_all_hashes == true)
+    {
+      // do not sort, because we need to keep all hashes in this particular case
+    }
+    else if (hashconfig->is_salted)
     {
       if (sort_by_salt (hashes_buf[hashes_pos].salt, hashes_buf[hashes_pos - 1].salt) == 0)
       {
@@ -1051,9 +1207,10 @@ int hashes_init_stage2 (hashcat_ctx_t *hashcat_ctx)
    * Now generate all the buffers required for later
    */
 
-  void   *digests_buf_new = (void *) hccalloc (hashes_cnt, hashconfig->dgst_size);
-  salt_t *salts_buf_new   = NULL;
-  void   *esalts_buf_new  = NULL;
+  void   *digests_buf_new    = (void *) hccalloc (hashes_cnt, hashconfig->dgst_size);
+  salt_t *salts_buf_new      = NULL;
+  void   *esalts_buf_new     = NULL;
+  void   *hook_salts_buf_new = NULL;
 
   if (hashconfig->is_salted)
   {
@@ -1069,6 +1226,11 @@ int hashes_init_stage2 (hashcat_ctx_t *hashcat_ctx)
     esalts_buf_new = (void *) hccalloc (hashes_cnt, hashconfig->esalt_size);
   }
 
+  if (hashconfig->hook_salt_size)
+  {
+    hook_salts_buf_new = (void *) hccalloc (hashes_cnt, hashconfig->hook_salt_size);
+  }
+
   EVENT (EVENT_HASHLIST_SORT_SALT_PRE);
 
   u32 digests_cnt  = hashes_cnt;
@@ -1082,21 +1244,9 @@ int hashes_init_stage2 (hashcat_ctx_t *hashcat_ctx)
 
   hashinfo_t **hash_info = NULL;
 
-  if ((user_options->username == true) || (hashconfig->opts_type & OPTS_TYPE_HASH_COPY))
+  if ((user_options->username == true) || (hashconfig->opts_type & OPTS_TYPE_HASH_COPY) || (hashconfig->opts_type & OPTS_TYPE_HASH_SPLIT))
   {
     hash_info = (hashinfo_t **) hccalloc (hashes_cnt, sizeof (hashinfo_t *));
-
-    if (user_options->username == true)
-    {
-      u32 user_pos;
-
-      for (user_pos = 0; user_pos < hashes_cnt; user_pos++)
-      {
-        hash_info[user_pos] = (hashinfo_t *) hcmalloc (sizeof (hashinfo_t));
-
-        hash_info[user_pos]->user = (user_t *) hcmalloc (sizeof (user_t));
-      }
-    }
   }
 
   u32 *salts_shown = (u32 *) hccalloc (digests_cnt, sizeof (u32));
@@ -1112,13 +1262,13 @@ int hashes_init_stage2 (hashcat_ctx_t *hashcat_ctx)
 
     hashes_buf[0].salt = salt_buf;
 
-    if (hashconfig->esalt_size)
+    if (hashconfig->hook_salt_size)
     {
-      char *esalts_buf_new_ptr = ((char *) esalts_buf_new) + (salts_cnt * hashconfig->esalt_size);
+      char *hook_salts_buf_new_ptr = ((char *) hook_salts_buf_new) + (salts_cnt * hashconfig->hook_salt_size);
 
-      memcpy (esalts_buf_new_ptr, hashes_buf[0].esalt, hashconfig->esalt_size);
+      memcpy (hook_salts_buf_new_ptr, hashes_buf[0].hook_salt, hashconfig->hook_salt_size);
 
-      hashes_buf[0].esalt = esalts_buf_new_ptr;
+      hashes_buf[0].hook_salt = hook_salts_buf_new_ptr;
     }
 
     salt_buf->digests_cnt    = 0;
@@ -1136,7 +1286,16 @@ int hashes_init_stage2 (hashcat_ctx_t *hashcat_ctx)
 
   hashes_buf[0].digest = digests_buf_new_ptr;
 
-  if ((user_options->username == true) || (hashconfig->opts_type & OPTS_TYPE_HASH_COPY))
+  if (hashconfig->esalt_size)
+  {
+    char *esalts_buf_new_ptr = ((char *) esalts_buf_new) + (0 * hashconfig->esalt_size);
+
+    memcpy (esalts_buf_new_ptr, hashes_buf[0].esalt, hashconfig->esalt_size);
+
+    hashes_buf[0].esalt = esalts_buf_new_ptr;
+  }
+
+  if ((user_options->username == true) || (hashconfig->opts_type & OPTS_TYPE_HASH_COPY) || (hashconfig->opts_type & OPTS_TYPE_HASH_SPLIT))
   {
     hash_info[0] = hashes_buf[0].hash_info;
   }
@@ -1155,13 +1314,13 @@ int hashes_init_stage2 (hashcat_ctx_t *hashcat_ctx)
 
         hashes_buf[hashes_pos].salt = salt_buf;
 
-        if (hashconfig->esalt_size)
+        if (hashconfig->hook_salt_size)
         {
-          char *esalts_buf_new_ptr = ((char *) esalts_buf_new) + (salts_cnt * hashconfig->esalt_size);
+          char *hook_salts_buf_new_ptr = ((char *) hook_salts_buf_new) + (salts_cnt * hashconfig->hook_salt_size);
 
-          memcpy (esalts_buf_new_ptr, hashes_buf[hashes_pos].esalt, hashconfig->esalt_size);
+          memcpy (hook_salts_buf_new_ptr, hashes_buf[hashes_pos].hook_salt, hashconfig->hook_salt_size);
 
-          hashes_buf[hashes_pos].esalt = esalts_buf_new_ptr;
+          hashes_buf[hashes_pos].hook_salt = hook_salts_buf_new_ptr;
         }
 
         salt_buf->digests_cnt    = 0;
@@ -1173,11 +1332,11 @@ int hashes_init_stage2 (hashcat_ctx_t *hashcat_ctx)
 
       hashes_buf[hashes_pos].salt = salt_buf;
 
-      if (hashconfig->esalt_size)
+      if (hashconfig->hook_salt_size)
       {
-        char *esalts_buf_new_ptr = ((char *) esalts_buf_new) + (salts_cnt * hashconfig->esalt_size);
+        char *hook_salts_buf_new_ptr = ((char *) hook_salts_buf_new) + (salts_cnt * hashconfig->hook_salt_size);
 
-        hashes_buf[hashes_pos].esalt = esalts_buf_new_ptr;
+        hashes_buf[hashes_pos].hook_salt = hook_salts_buf_new_ptr;
       }
     }
 
@@ -1189,7 +1348,16 @@ int hashes_init_stage2 (hashcat_ctx_t *hashcat_ctx)
 
     hashes_buf[hashes_pos].digest = digests_buf_new_ptr;
 
-    if ((user_options->username == true) || (hashconfig->opts_type & OPTS_TYPE_HASH_COPY))
+    if (hashconfig->esalt_size)
+    {
+      char *esalts_buf_new_ptr = ((char *) esalts_buf_new) + (hashes_pos * hashconfig->esalt_size);
+
+      memcpy (esalts_buf_new_ptr, hashes_buf[hashes_pos].esalt, hashconfig->esalt_size);
+
+      hashes_buf[hashes_pos].esalt = esalts_buf_new_ptr;
+    }
+
+    if ((user_options->username == true) || (hashconfig->opts_type & OPTS_TYPE_HASH_COPY) || (hashconfig->opts_type & OPTS_TYPE_HASH_SPLIT))
     {
       hash_info[hashes_pos] = hashes_buf[hashes_pos].hash_info;
     }
@@ -1200,21 +1368,23 @@ int hashes_init_stage2 (hashcat_ctx_t *hashcat_ctx)
   hcfree (hashes->digests_buf);
   hcfree (hashes->salts_buf);
   hcfree (hashes->esalts_buf);
+  hcfree (hashes->hook_salts_buf);
 
-  hashes->digests_cnt        = digests_cnt;
-  hashes->digests_done       = digests_done;
-  hashes->digests_buf        = digests_buf_new;
-  hashes->digests_shown      = digests_shown;
-  hashes->digests_shown_tmp  = digests_shown_tmp;
+  hashes->digests_cnt       = digests_cnt;
+  hashes->digests_done      = digests_done;
+  hashes->digests_buf       = digests_buf_new;
+  hashes->digests_shown     = digests_shown;
+  hashes->digests_shown_tmp = digests_shown_tmp;
 
-  hashes->salts_cnt          = salts_cnt;
-  hashes->salts_done         = salts_done;
-  hashes->salts_buf          = salts_buf_new;
-  hashes->salts_shown        = salts_shown;
+  hashes->salts_cnt         = salts_cnt;
+  hashes->salts_done        = salts_done;
+  hashes->salts_buf         = salts_buf_new;
+  hashes->salts_shown       = salts_shown;
 
-  hashes->esalts_buf         = esalts_buf_new;
+  hashes->esalts_buf        = esalts_buf_new;
+  hashes->hook_salts_buf    = hook_salts_buf_new;
 
-  hashes->hash_info          = hash_info;
+  hashes->hash_info         = hash_info;
 
   return 0;
 }
@@ -1342,19 +1512,14 @@ int hashes_init_stage4 (hashcat_ctx_t *hashcat_ctx)
 
   hashes->tmp_buf = tmp_buf;
 
-  // special wpa booster case
-
-  if (hashconfig->hash_mode == 2500)
-  {
-    wpa_essid_reuse (hashcat_ctx);
-  }
-
   return 0;
 }
 
 void hashes_destroy (hashcat_ctx_t *hashcat_ctx)
 {
-  hashes_t *hashes = hashcat_ctx->hashes;
+  hashconfig_t   *hashconfig   = hashcat_ctx->hashconfig;
+  user_options_t *user_options = hashcat_ctx->user_options;
+  hashes_t *hashes             = hashcat_ctx->hashes;
 
   hcfree (hashes->digests_buf);
   hcfree (hashes->digests_shown);
@@ -1363,10 +1528,32 @@ void hashes_destroy (hashcat_ctx_t *hashcat_ctx)
   hcfree (hashes->salts_buf);
   hcfree (hashes->salts_shown);
 
-  hcfree (hashes->esalts_buf);
+  if ((user_options->username == true) || (hashconfig->opts_type & OPTS_TYPE_HASH_COPY))
+  {
+    for (u32 hash_pos = 0; hash_pos < hashes->hashes_cnt; hash_pos++)
+    {
+      if (user_options->username == true)
+      {
+        hcfree (hashes->hash_info[hash_pos]->user);
+      }
+
+      if (hashconfig->opts_type & OPTS_TYPE_HASH_COPY)
+      {
+        hcfree (hashes->hash_info[hash_pos]->orighash);
+      }
+
+      if (hashconfig->opts_type & OPTS_TYPE_HASH_SPLIT)
+      {
+        hcfree (hashes->hash_info[hash_pos]->split);
+      }
+    }
+  }
 
   hcfree (hashes->hash_info);
 
+  hcfree (hashes->esalts_buf);
+  hcfree (hashes->hook_salts_buf);
+
   hcfree (hashes->out_buf);
   hcfree (hashes->tmp_buf);
 
