@@ -16,8 +16,14 @@ static int sort_by_mtime (const void *p1, const void *p2)
   const char **f1 = (const char **) p1;
   const char **f2 = (const char **) p2;
 
-  hc_stat_t s1; hc_stat (*f1, &s1);
-  hc_stat_t s2; hc_stat (*f2, &s2);
+  hc_stat_t s1;
+  hc_stat_t s2;
+
+  const int rc1 = hc_stat (*f1, &s1);
+  const int rc2 = hc_stat (*f2, &s2);
+
+  if (rc1 < rc2) return  1;
+  if (rc1 > rc2) return -1;
 
   return s2.st_mtime - s1.st_mtime;
 }
@@ -66,7 +72,7 @@ int induct_ctx_init (hashcat_ctx_t *hashcat_ctx)
 
         if (rename (root_directory, root_directory_mv) != 0)
         {
-          event_log_error (hashcat_ctx, "Rename directory %s to %s: %m", root_directory, root_directory_mv);
+          event_log_error (hashcat_ctx, "Rename directory %s to %s: %s", root_directory, root_directory_mv, strerror (errno));
 
           return -1;
         }
@@ -75,7 +81,7 @@ int induct_ctx_init (hashcat_ctx_t *hashcat_ctx)
       }
       else
       {
-        event_log_error (hashcat_ctx, "%s: %m", root_directory);
+        event_log_error (hashcat_ctx, "%s: %s", root_directory, strerror (errno));
 
         return -1;
       }
@@ -83,7 +89,7 @@ int induct_ctx_init (hashcat_ctx_t *hashcat_ctx)
 
     if (hc_mkdir (root_directory, 0700) == -1)
     {
-      event_log_error (hashcat_ctx, "%s: %m", root_directory);
+      event_log_error (hashcat_ctx, "%s: %s", root_directory, strerror (errno));
 
       return -1;
     }
@@ -129,7 +135,7 @@ void induct_ctx_destroy (hashcat_ctx_t *hashcat_ctx)
     }
     else
     {
-      event_log_error (hashcat_ctx, "%s: %m", induct_ctx->root_directory);
+      event_log_error (hashcat_ctx, "%s: %s", induct_ctx->root_directory, strerror (errno));
 
       //return -1;
     }
