@@ -108,7 +108,7 @@ int loopback_write_open (hashcat_ctx_t *hashcat_ctx)
 
   if (fp == NULL)
   {
-    event_log_error (hashcat_ctx, "%s: %m", loopback_ctx->filename);
+    event_log_error (hashcat_ctx, "%s: %s", loopback_ctx->filename, strerror (errno));
 
     return -1;
   }
@@ -163,7 +163,10 @@ void loopback_write_append (hashcat_ctx_t *hashcat_ctx, const u8 *plain_ptr, con
 
   fflush (fp);
 
-  unlock_file (fp);
+  if (unlock_file (fp))
+  {
+    event_log_error (hashcat_ctx, "%s: Failed to unlock file", loopback_ctx->filename);
+  }
 
   loopback_ctx->unused = false;
 }
