@@ -20,106 +20,6 @@ static void fsync (int fd)
 }
 #endif
 
-static int check_running_process (hashcat_ctx_t *hashcat_ctx)
-{
-  restore_ctx_t *restore_ctx = hashcat_ctx->restore_ctx;
-
-  char *eff_restore_file = restore_ctx->eff_restore_file;
-
-  FILE *fp = fopen (eff_restore_file, "rb");
-
-  if (fp == NULL) return 0;
-
-  restore_data_t *rd = (restore_data_t *) hcmalloc (sizeof (restore_data_t));
-
-  const size_t nread = fread (rd, sizeof (restore_data_t), 1, fp);
-
-  fclose (fp);
-
-  if (nread != 1)
-  {
-    event_log_error (hashcat_ctx, "Cannot read %s", eff_restore_file);
-
-    return -1;
-  }
-
-  if (rd->pid)
-  {
-    #if defined (_POSIX)
-
-    char *pidbin;
-
-    hc_asprintf (&pidbin, "/proc/%u/cmdline", rd->pid);
-
-    FILE *fd = fopen (pidbin, "rb");
-
-    if (fd)
-    {
-      size_t pidbin_len = fread (pidbin, 1, HCBUFSIZ_LARGE, fd);
-
-      pidbin[pidbin_len] = 0;
-
-      fclose (fd);
-
-      char *argv0_r = strrchr (restore_ctx->argv[0], '/');
-
-      char *pidbin_r = strrchr (pidbin, '/');
-
-      if (argv0_r == NULL) argv0_r = restore_ctx->argv[0];
-
-      if (pidbin_r == NULL) pidbin_r = pidbin;
-
-      if (strcmp (argv0_r, pidbin_r) == 0)
-      {
-        event_log_error (hashcat_ctx, "Already an instance %s running on pid %u", pidbin, rd->pid);
-
-        return -1;
-      }
-    }
-
-    hcfree (pidbin);
-
-    #elif defined (_WIN)
-
-    HANDLE hProcess = OpenProcess (PROCESS_ALL_ACCESS, FALSE, rd->pid);
-
-    char *pidbin  = (char *) hcmalloc (HCBUFSIZ_LARGE);
-    char *pidbin2 = (char *) hcmalloc (HCBUFSIZ_LARGE);
-
-    int pidbin_len  = GetModuleFileName (NULL, pidbin, HCBUFSIZ_LARGE);
-    int pidbin2_len = GetModuleFileNameEx (hProcess, NULL, pidbin2, HCBUFSIZ_LARGE);
-
-    pidbin[pidbin_len]   = 0;
-    pidbin2[pidbin2_len] = 0;
-
-    if (pidbin2_len)
-    {
-      if (strcmp (pidbin, pidbin2) == 0)
-      {
-        event_log_error (hashcat_ctx, "Already an instance %s running on pid %d", pidbin2, rd->pid);
-
-        return -1;
-      }
-    }
-
-    hcfree (pidbin2);
-    hcfree (pidbin);
-
-    #endif
-  }
-
-  if (rd->version < RESTORE_VERSION_MIN)
-  {
-    event_log_error (hashcat_ctx, "Cannot use outdated %s. Please remove it.", eff_restore_file);
-
-    return -1;
-  }
-
-  hcfree (rd);
-
-  return 0;
-}
-
 static int init_restore (hashcat_ctx_t *hashcat_ctx)
 {
   restore_ctx_t *restore_ctx = hashcat_ctx->restore_ctx;
@@ -128,24 +28,14 @@ static int init_restore (hashcat_ctx_t *hashcat_ctx)
 
   restore_ctx->rd = rd;
 
-  const int rc = check_running_process (hashcat_ctx);
-
-  if (rc == -1) return -1;
-
   rd->version = RESTORE_VERSION_CUR;
 
   rd->argc = restore_ctx->argc;
   rd->argv = restore_ctx->argv;
 
-  #if defined (_POSIX)
-  rd->pid = getpid ();
-  #elif defined (_WIN)
-  rd->pid = GetCurrentProcessId ();
-  #endif
-
   if (getcwd (rd->cwd, 255) == NULL)
   {
-    event_log_error (hashcat_ctx, "getcwd(): %m");
+    event_log_error (hashcat_ctx, "getcwd(): %s", strerror (errno));
 
     return -1;
   }
@@ -165,7 +55,7 @@ static int read_restore (hashcat_ctx_t *hashcat_ctx)
 
   if (fp == NULL)
   {
-    event_log_error (hashcat_ctx, "Restore file '%s': %m", eff_restore_file);
+    event_log_error (hashcat_ctx, "Restore file '%s': %s", eff_restore_file, strerror (errno));
 
     return -1;
   }
@@ -174,7 +64,27 @@ static int read_restore (hashcat_ctx_t *hashcat_ctx)
 
   if (fread (rd, sizeof (restore_data_t), 1, fp) != 1)
   {
-    event_log_error (hashcat_ctx, "Can't read %s", eff_restore_file);
+    event_log_error (hashcat_ctx, "Cannot read %s", eff_restore_file);
+
+    fclose (fp);
+
+    return -1;
+  }
+
+  // we only use these 2 checks to avoid "tainted string" warnings
+
+  if (rd->argc < 1)
+  {
+    event_log_error (hashcat_ctx, "Unusually low number of arguments (argc) within restore file %s", eff_restore_file);
+
+    fclose (fp);
+
+    return -1;
+  }
+
+  if (rd->argc > 250) // some upper bound check is always good (with some dirs/dicts it could be a large string)
+  {
+    event_log_error (hashcat_ctx, "Unusually high number of arguments (argc) within restore file %s", eff_restore_file);
 
     fclose (fp);
 
@@ -189,7 +99,7 @@ static int read_restore (hashcat_ctx_t *hashcat_ctx)
   {
     if (fgets (buf, HCBUFSIZ_LARGE - 1, fp) == NULL)
     {
-      event_log_error (hashcat_ctx, "Can't read %s", eff_restore_file);
+      event_log_error (hashcat_ctx, "Cannot read %s", eff_restore_file);
 
       fclose (fp);
 
@@ -207,15 +117,32 @@ static int read_restore (hashcat_ctx_t *hashcat_ctx)
 
   fclose (fp);
 
+  if (hc_path_exist (rd->cwd) == false)
+  {
+    event_log_error (hashcat_ctx, "%s: %s", rd->cwd, strerror (errno));
+
+    return -1;
+  }
+
+  if (hc_path_is_directory (rd->cwd) == false)
+  {
+    event_log_error (hashcat_ctx, "%s: %s", rd->cwd, strerror (errno));
+
+    return -1;
+  }
+
   event_log_warning (hashcat_ctx, "Changing current working directory to '%s'", rd->cwd);
-  event_log_warning (hashcat_ctx, "");
+  event_log_warning (hashcat_ctx, NULL);
 
   if (chdir (rd->cwd))
   {
-    event_log_error (hashcat_ctx, "The directory '%s' does not exist. It is needed to restore (--restore) the session.", rd->cwd);
-    event_log_error (hashcat_ctx, "You could either create this directory or update the .restore file using e.g. the analyze_hc_restore.pl tool:");
-    event_log_error (hashcat_ctx, "https://github.com/philsmd/analyze_hc_restore");
-    event_log_error (hashcat_ctx, "The directory must contain all files and folders mentioned within the command line.");
+    event_log_error (hashcat_ctx, "Directory '%s' needed to restore the session was not found.", rd->cwd);
+
+    event_log_warning (hashcat_ctx, "Either create the directory, or update the directory within the .restore file.");
+    event_log_warning (hashcat_ctx, "Restore files can be analyzed and modified with analyze_hc_restore.pl:");
+    event_log_warning (hashcat_ctx, "    https://github.com/philsmd/analyze_hc_restore");
+    event_log_warning (hashcat_ctx, "Directory must contain all files and folders from the original command line.");
+    event_log_warning (hashcat_ctx, NULL);
 
     return -1;
   }
@@ -244,14 +171,14 @@ static int write_restore (hashcat_ctx_t *hashcat_ctx)
 
   if (fp == NULL)
   {
-    event_log_error (hashcat_ctx, "%s: %m", new_restore_file);
+    event_log_error (hashcat_ctx, "%s: %s", new_restore_file, strerror (errno));
 
     return -1;
   }
 
   if (setvbuf (fp, NULL, _IONBF, 0))
   {
-    event_log_error (hashcat_ctx, "setvbuf file '%s': %m", new_restore_file);
+    event_log_error (hashcat_ctx, "setvbuf file '%s': %s", new_restore_file, strerror (errno));
 
     fclose (fp);
 
@@ -293,19 +220,17 @@ int cycle_restore (hashcat_ctx_t *hashcat_ctx)
 
   if (rc_write_restore == -1) return -1;
 
-  hc_stat_t st;
-
-  if (hc_stat (eff_restore_file, &st) == 0)
+  if (hc_path_exist (eff_restore_file) == true)
   {
-    if (unlink (eff_restore_file))
+    if (unlink (eff_restore_file) == -1)
     {
-      event_log_warning (hashcat_ctx, "Unlink file '%s': %m", eff_restore_file);
+      event_log_warning (hashcat_ctx, "Unlink file '%s': %s", eff_restore_file, strerror (errno));
     }
   }
 
-  if (rename (new_restore_file, eff_restore_file))
+  if (rename (new_restore_file, eff_restore_file) == -1)
   {
-    event_log_warning (hashcat_ctx, "Rename file '%s' to '%s': %m", new_restore_file, eff_restore_file);
+    event_log_warning (hashcat_ctx, "Rename file '%s' to '%s': %s", new_restore_file, eff_restore_file, strerror (errno));
   }
 
   return 0;
@@ -384,17 +309,11 @@ int restore_ctx_init (hashcat_ctx_t *hashcat_ctx, int argc, char **argv)
 
     if (rd->version < RESTORE_VERSION_MIN)
     {
-      event_log_error (hashcat_ctx, "Incompatible restore-file version");
+      event_log_error (hashcat_ctx, "Incompatible restore-file version.");
 
       return -1;
     }
 
-    #if defined (_POSIX)
-    rd->pid = getpid ();
-    #elif defined (_WIN)
-    rd->pid = GetCurrentProcessId ();
-    #endif
-
     user_options_init (hashcat_ctx);
 
     const int rc_options_getopt = user_options_getopt (hashcat_ctx, rd->argc, rd->argv);
