@@ -69,12 +69,19 @@ int straight_ctx_update_loop (hashcat_ctx_t *hashcat_ctx)
 
       if (fd == NULL)
       {
-        event_log_error (hashcat_ctx, "%s: %m", straight_ctx->dict);
+        event_log_error (hashcat_ctx, "%s: %s", straight_ctx->dict, strerror (errno));
 
         return -1;
       }
 
-      status_ctx->words_cnt = count_words (hashcat_ctx, fd, straight_ctx->dict);
+      const int rc = count_words (hashcat_ctx, fd, straight_ctx->dict, &status_ctx->words_cnt);
+
+      if (rc == -1)
+      {
+        event_log_error (hashcat_ctx, "Integer overflow detected in keyspace of wordlist: %s", straight_ctx->dict);
+
+        return -1;
+      }
 
       fclose (fd);
 
@@ -97,12 +104,19 @@ int straight_ctx_update_loop (hashcat_ctx_t *hashcat_ctx)
 
       if (fd == NULL)
       {
-        event_log_error (hashcat_ctx, "%s: %m", combinator_ctx->dict1);
+        event_log_error (hashcat_ctx, "%s: %s", combinator_ctx->dict1, strerror (errno));
 
         return -1;
       }
 
-      status_ctx->words_cnt = count_words (hashcat_ctx, fd, combinator_ctx->dict1);
+      const int rc = count_words (hashcat_ctx, fd, combinator_ctx->dict1, &status_ctx->words_cnt);
+
+      if (rc == -1)
+      {
+        event_log_error (hashcat_ctx, "Integer overflow detected in keyspace of wordlist: %s", combinator_ctx->dict1);
+
+        return -1;
+      }
 
       fclose (fd);
     }
@@ -112,12 +126,19 @@ int straight_ctx_update_loop (hashcat_ctx_t *hashcat_ctx)
 
       if (fd == NULL)
       {
-        event_log_error (hashcat_ctx, "%s: %m", combinator_ctx->dict2);
+        event_log_error (hashcat_ctx, "%s: %s", combinator_ctx->dict2, strerror (errno));
 
         return -1;
       }
 
-      status_ctx->words_cnt = count_words (hashcat_ctx, fd, combinator_ctx->dict2);
+      const int rc = count_words (hashcat_ctx, fd, combinator_ctx->dict2, &status_ctx->words_cnt);
+
+      if (rc == -1)
+      {
+        event_log_error (hashcat_ctx, "Integer overflow detected in keyspace of wordlist: %s", combinator_ctx->dict2);
+
+        return -1;
+      }
 
       fclose (fd);
     }
@@ -151,12 +172,19 @@ int straight_ctx_update_loop (hashcat_ctx_t *hashcat_ctx)
 
     if (fd == NULL)
     {
-      event_log_error (hashcat_ctx, "%s: %m", straight_ctx->dict);
+      event_log_error (hashcat_ctx, "%s: %s", straight_ctx->dict, strerror (errno));
 
       return -1;
     }
 
-    status_ctx->words_cnt = count_words (hashcat_ctx, fd, straight_ctx->dict);
+    const int rc = count_words (hashcat_ctx, fd, straight_ctx->dict, &status_ctx->words_cnt);
+
+    if (rc == -1)
+    {
+      event_log_error (hashcat_ctx, "Integer overflow detected in keyspace of wordlist: %s", straight_ctx->dict);
+
+      return -1;
+    }
 
     fclose (fd);
 
@@ -268,18 +296,11 @@ int straight_ctx_init (hashcat_ctx_t *hashcat_ctx)
       {
         char *l0_filename = user_options_extra->hc_workv[i];
 
-        hc_stat_t l0_stat;
+        // at this point we already verified the path actually exist and is readable
 
-        if (hc_stat (l0_filename, &l0_stat) == -1)
+        if (hc_path_is_directory (l0_filename) == true)
         {
-          event_log_error (hashcat_ctx, "%s: %m", l0_filename);
-
-          return -1;
-        }
-
-        if (S_ISDIR (l0_stat.st_mode))
-        {
-          char **dictionary_files = NULL;
+          char **dictionary_files;
 
           dictionary_files = scan_directory (l0_filename);
 
@@ -291,20 +312,25 @@ int straight_ctx_init (hashcat_ctx_t *hashcat_ctx)
             {
               char *l1_filename = dictionary_files[d];
 
-              hc_stat_t l1_stat;
-
-              if (hc_stat (l1_filename, &l1_stat) == -1)
+              if (hc_path_read (l1_filename) == false)
               {
-                event_log_error (hashcat_ctx, "%s: %m", l1_filename);
+                event_log_error (hashcat_ctx, "%s: %s", l1_filename, strerror (errno));
+
+                hcfree (dictionary_files);
 
                 return -1;
               }
 
-              if (S_ISREG (l1_stat.st_mode))
+              if (hc_path_is_file (l1_filename) == true)
               {
                 const int rc = straight_ctx_add_wl (hashcat_ctx, l1_filename);
 
-                if (rc == -1) return -1;
+                if (rc == -1)
+                {
+                  hcfree (dictionary_files);
+
+                  return -1;
+                }
               }
             }
           }
@@ -341,18 +367,11 @@ int straight_ctx_init (hashcat_ctx_t *hashcat_ctx)
     {
       char *l0_filename = user_options_extra->hc_workv[i];
 
-      hc_stat_t l0_stat;
+      // at this point we already verified the path actually exist and is readable
 
-      if (hc_stat (l0_filename, &l0_stat) == -1)
+      if (hc_path_is_directory (l0_filename) == true)
       {
-        event_log_error (hashcat_ctx, "%s: %m", l0_filename);
-
-        return -1;
-      }
-
-      if (S_ISDIR (l0_stat.st_mode))
-      {
-        char **dictionary_files = NULL;
+        char **dictionary_files;
 
         dictionary_files = scan_directory (l0_filename);
 
@@ -364,20 +383,25 @@ int straight_ctx_init (hashcat_ctx_t *hashcat_ctx)
           {
             char *l1_filename = dictionary_files[d];
 
-            hc_stat_t l1_stat;
-
-            if (hc_stat (l1_filename, &l1_stat) == -1)
+            if (hc_path_read (l1_filename) == false)
             {
-              event_log_error (hashcat_ctx, "%s: %m", l1_filename);
+              event_log_error (hashcat_ctx, "%s: %s", l1_filename, strerror (errno));
+
+              hcfree (dictionary_files);
 
               return -1;
             }
 
-            if (S_ISREG (l1_stat.st_mode))
+            if (hc_path_is_file (l1_filename) == true)
             {
               const int rc = straight_ctx_add_wl (hashcat_ctx, l1_filename);
 
-              if (rc == -1) return -1;
+              if (rc == -1)
+              {
+                hcfree (dictionary_files);
+
+                return -1;
+              }
             }
           }
         }
@@ -405,18 +429,11 @@ int straight_ctx_init (hashcat_ctx_t *hashcat_ctx)
     {
       char *l0_filename = user_options_extra->hc_workv[i];
 
-      hc_stat_t l0_stat;
+      // at this point we already verified the path actually exist and is readable
 
-      if (hc_stat (l0_filename, &l0_stat) == -1)
+      if (hc_path_is_directory (l0_filename) == true)
       {
-        event_log_error (hashcat_ctx, "%s: %m", l0_filename);
-
-        return -1;
-      }
-
-      if (S_ISDIR (l0_stat.st_mode))
-      {
-        char **dictionary_files = NULL;
+        char **dictionary_files;
 
         dictionary_files = scan_directory (l0_filename);
 
@@ -428,20 +445,25 @@ int straight_ctx_init (hashcat_ctx_t *hashcat_ctx)
           {
             char *l1_filename = dictionary_files[d];
 
-            hc_stat_t l1_stat;
-
-            if (hc_stat (l1_filename, &l1_stat) == -1)
+            if (hc_path_read (l1_filename) == false)
             {
-              event_log_error (hashcat_ctx, "%s: %m", l1_filename);
+              event_log_error (hashcat_ctx, "%s: %s", l1_filename, strerror (errno));
+
+              hcfree (dictionary_files);
 
               return -1;
             }
 
-            if (S_ISREG (l1_stat.st_mode))
+            if (hc_path_is_file (l1_filename) == true)
             {
               const int rc = straight_ctx_add_wl (hashcat_ctx, l1_filename);
 
-              if (rc == -1) return -1;
+              if (rc == -1)
+              {
+                hcfree (dictionary_files);
+
+                return -1;
+              }
             }
           }
         }
