@@ -76,7 +76,7 @@ int tuning_db_init (hashcat_ctx_t *hashcat_ctx)
 
   if (fp == NULL)
   {
-    event_log_error (hashcat_ctx, "%s: %m", tuning_db_file);
+    event_log_error (hashcat_ctx, "%s: %s", tuning_db_file, strerror (errno));
 
     return -1;
   }
@@ -119,7 +119,7 @@ int tuning_db_init (hashcat_ctx_t *hashcat_ctx)
 
     int token_cnt = 0;
 
-    char *saveptr = NULL;
+    char *saveptr;
 
     char *next = strtok_r (line_buf, "\t ", &saveptr);
 
@@ -266,7 +266,7 @@ void tuning_db_destroy (hashcat_ctx_t *hashcat_ctx)
   {
     tuning_db_entry_t *entry = &tuning_db->entry_buf[i];
 
-    hcfree (entry->device_name);
+    hcfree ((void *)entry->device_name);
   }
 
   hcfree (tuning_db->alias_buf);
