@@ -34,27 +34,11 @@ u32 convert_from_hex (hashcat_ctx_t *hashcat_ctx, char *line_buf, const u32 line
     return (i);
   }
 
-  if (line_len >= 6) // $HEX[] = 6
+  if (is_hexify ((const u8 *) line_buf, (const int) line_len) == true)
   {
-    if (line_buf[0]            != '$') return (line_len);
-    if (line_buf[1]            != 'H') return (line_len);
-    if (line_buf[2]            != 'E') return (line_len);
-    if (line_buf[3]            != 'X') return (line_len);
-    if (line_buf[4]            != '[') return (line_len);
-    if (line_buf[line_len - 1] != ']') return (line_len);
+    const int new_len = exec_unhexify ((const u8 *) line_buf, (const int) line_len, (u8 *) line_buf, (const int) line_len);
 
-    size_t i, j;
-
-    for (i = 0, j = 5; j < line_len - 1; i += 1, j += 2)
-    {
-      const u8 c = hex_to_u8 ((const u8 *) &line_buf[j]);
-
-      line_buf[i] = c;
-    }
-
-    memset (line_buf + i, 0, line_len - i);
-
-    return (i);
+    return (u32) new_len;
   }
 
   return (line_len);
@@ -200,6 +184,28 @@ void get_next_word (hashcat_ctx_t *hashcat_ctx, FILE *fd, char **out_buf, u32 *o
 
     wl_data->pos += off;
 
+    // do the on-the-fly encoding
+
+    if (wl_data->iconv_enabled == true)
+    {
+      char  *iconv_ptr = wl_data->iconv_tmp;
+      size_t iconv_sz  = HCBUFSIZ_TINY;
+
+      size_t ptr_len = len;
+
+      const size_t iconv_rc = iconv (wl_data->iconv_ctx, &ptr, &ptr_len, &iconv_ptr, &iconv_sz);
+
+      if (iconv_rc == (size_t) -1)
+      {
+        len = PW_MAX1;
+      }
+      else
+      {
+        ptr = wl_data->iconv_tmp;
+        len = HCBUFSIZ_TINY - iconv_sz;
+      }
+    }
+
     if (run_rule_engine (user_options_extra->rule_len_l, user_options->rule_buf_l))
     {
       int rule_len_out = -1;
@@ -271,7 +277,7 @@ void pw_add (hc_device_param_t *device_param, const u8 *pw_buf, const int pw_len
   //}
 }
 
-u64 count_words (hashcat_ctx_t *hashcat_ctx, FILE *fd, const char *dictfile)
+int count_words (hashcat_ctx_t *hashcat_ctx, FILE *fd, const char *dictfile, u64 *result)
 {
   combinator_ctx_t     *combinator_ctx     = hashcat_ctx->combinator_ctx;
   straight_ctx_t       *straight_ctx       = hashcat_ctx->straight_ctx;
@@ -285,7 +291,12 @@ u64 count_words (hashcat_ctx_t *hashcat_ctx, FILE *fd, const char *dictfile)
 
   d.cnt = 0;
 
-  hc_fstat (fileno (fd), &d.stat);
+  if (hc_fstat (fileno (fd), &d.stat))
+  {
+    *result = 0;
+
+    return 0;
+  }
 
   d.stat.st_mode    = 0;
   d.stat.st_nlink   = 0;
@@ -299,7 +310,18 @@ u64 count_words (hashcat_ctx_t *hashcat_ctx, FILE *fd, const char *dictfile)
   d.stat.st_blocks  = 0;
   #endif
 
-  if (d.stat.st_size == 0) return 0;
+  memset (d.encoding_from, 0, sizeof (d.encoding_from));
+  memset (d.encoding_to,   0, sizeof (d.encoding_to));
+
+  strncpy (d.encoding_from, user_options->encoding_from, sizeof (d.encoding_from));
+  strncpy (d.encoding_to,   user_options->encoding_to,   sizeof (d.encoding_to));
+
+  if (d.stat.st_size == 0)
+  {
+    *result = 0;
+
+    return 0;
+  }
 
   const u64 cached_cnt = dictstat_find (hashcat_ctx, &d);
 
@@ -311,10 +333,14 @@ u64 count_words (hashcat_ctx_t *hashcat_ctx, FILE *fd, const char *dictfile)
 
       if (user_options_extra->attack_kern == ATTACK_KERN_STRAIGHT)
       {
+        if (overflow_check_u64_mul (keyspace, straight_ctx->kernel_rules_cnt) == false) return -1;
+
         keyspace *= straight_ctx->kernel_rules_cnt;
       }
       else if (user_options_extra->attack_kern == ATTACK_KERN_COMBI)
       {
+        if (overflow_check_u64_mul (keyspace, combinator_ctx->combs_cnt) == false) return -1;
+
         keyspace *= combinator_ctx->combs_cnt;
       }
 
@@ -327,10 +353,16 @@ u64 count_words (hashcat_ctx_t *hashcat_ctx, FILE *fd, const char *dictfile)
 
       EVENT_DATA (EVENT_WORDLIST_CACHE_HIT, &cache_hit, sizeof (cache_hit));
 
-      return (keyspace);
+      *result = keyspace;
+
+      return 0;
     }
   }
 
+  time_t rt_start;
+
+  time (&rt_start);
+
   time_t now  = 0;
   time_t prev = 0;
 
@@ -351,7 +383,31 @@ u64 count_words (hashcat_ctx_t *hashcat_ctx, FILE *fd, const char *dictfile)
       u64 len;
       u64 off;
 
-      wl_data->func (wl_data->buf + i, wl_data->cnt - i, &len, &off);
+      char *ptr = wl_data->buf + i;
+
+      wl_data->func (ptr, wl_data->cnt - i, &len, &off);
+
+      // do the on-the-fly encoding
+
+      if (wl_data->iconv_enabled == true)
+      {
+        char  *iconv_ptr = wl_data->iconv_tmp;
+        size_t iconv_sz  = HCBUFSIZ_TINY;
+
+        size_t ptr_len = len;
+
+        const size_t iconv_rc = iconv (wl_data->iconv_ctx, &ptr, &ptr_len, &iconv_ptr, &iconv_sz);
+
+        if (iconv_rc == (size_t) -1)
+        {
+          len = PW_MAX1;
+        }
+        else
+        {
+          ptr = wl_data->iconv_tmp;
+          len = HCBUFSIZ_TINY - iconv_sz;
+        }
+      }
 
       if (run_rule_engine (user_options_extra->rule_len_l, user_options->rule_buf_l))
       {
@@ -361,7 +417,7 @@ u64 count_words (hashcat_ctx_t *hashcat_ctx, FILE *fd, const char *dictfile)
         {
           char unused[BLOCK_SIZE] = { 0 };
 
-          rule_len_out = _old_apply_rule (user_options->rule_buf_l, user_options_extra->rule_len_l, wl_data->buf + i, len, unused);
+          rule_len_out = _old_apply_rule (user_options->rule_buf_l, user_options_extra->rule_len_l, ptr, len, unused);
         }
 
         if (rule_len_out < 0)
@@ -378,10 +434,14 @@ u64 count_words (hashcat_ctx_t *hashcat_ctx, FILE *fd, const char *dictfile)
       {
         if (user_options_extra->attack_kern == ATTACK_KERN_STRAIGHT)
         {
+          if (overflow_check_u64_add (cnt, straight_ctx->kernel_rules_cnt) == false) return -1;
+
           cnt += straight_ctx->kernel_rules_cnt;
         }
         else if (user_options_extra->attack_kern == ATTACK_KERN_COMBI)
         {
+          if (overflow_check_u64_add (cnt, combinator_ctx->combs_cnt) == false) return -1;
+
           cnt += combinator_ctx->combs_cnt;
         }
 
@@ -415,6 +475,10 @@ u64 count_words (hashcat_ctx_t *hashcat_ctx, FILE *fd, const char *dictfile)
     }
   }
 
+  time_t rt_stop;
+
+  time (&rt_stop);
+
   cache_generate_t cache_generate;
 
   cache_generate.dictfile    = (char *) dictfile;
@@ -422,6 +486,7 @@ u64 count_words (hashcat_ctx_t *hashcat_ctx, FILE *fd, const char *dictfile)
   cache_generate.percent     = 100;
   cache_generate.cnt         = cnt;
   cache_generate.cnt2        = cnt2;
+  cache_generate.runtime     = rt_stop - rt_start;
 
   EVENT_DATA (EVENT_WORDLIST_CACHE_GENERATE, &cache_generate, sizeof (cache_generate));
 
@@ -429,7 +494,9 @@ u64 count_words (hashcat_ctx_t *hashcat_ctx, FILE *fd, const char *dictfile)
 
   //hc_signal (sigHandler_default);
 
-  return (cnt);
+  *result = cnt;
+
+  return 0;
 }
 
 int wl_data_init (hashcat_ctx_t *hashcat_ctx)
@@ -470,6 +537,21 @@ int wl_data_init (hashcat_ctx_t *hashcat_ctx)
     wl_data->func = get_next_word_lm;
   }
 
+  /**
+   * iconv
+   */
+
+  if (strcmp (user_options->encoding_from, user_options->encoding_to))
+  {
+    wl_data->iconv_enabled = true;
+
+    wl_data->iconv_ctx = iconv_open (user_options->encoding_to, user_options->encoding_from);
+
+    if (wl_data->iconv_ctx == (iconv_t) -1) return -1;
+
+    wl_data->iconv_tmp = (char *) hcmalloc (HCBUFSIZ_TINY);
+  }
+
   return 0;
 }
 
@@ -481,5 +563,14 @@ void wl_data_destroy (hashcat_ctx_t *hashcat_ctx)
 
   hcfree (wl_data->buf);
 
+  if (wl_data->iconv_enabled == true)
+  {
+    iconv_close (wl_data->iconv_ctx);
+
+    wl_data->iconv_enabled = false;
+
+    hcfree (wl_data->iconv_tmp);
+  }
+
   memset (wl_data, 0, sizeof (wl_data_t));
 }
