@@ -9,7 +9,7 @@ TDIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
 
 # missing hash types: 5200,6251,6261,6271,6281
 
-HASH_TYPES="0 10 11 12 20 21 22 23 30 40 50 60 100 101 110 111 112 120 121 122 125 130 131 132 133 140 141 150 160 200 300 400 500 900 1000 1100 1300 1400 1410 1420 1430 1440 1441 1450 1460 1500 1600 1700 1710 1711 1720 1722 1730 1731 1740 1750 1760 1800 2100 2400 2410 2500 2600 2611 2612 2711 2811 3000 3100 3200 3710 3711 3800 4300 4400 4500 4700 4800 4900 5000 5100 5300 5400 5500 5600 5700 5800 6000 6100 6211 6212 6213 6221 6222 6223 6231 6232 6233 6241 6242 6243 6300 6400 6500 6600 6700 6800 6900 7100 7200 7300 7400 7500 7600 7700 7800 7900 8000 8100 8200 8300 8400 8500 8600 8700 8900 9100 9200 9300 9400 9500 9600 9700 9800 9900 10000 10100 10200 10300 10400 10500 10600 10700 10800 10900 11000 11100 11200 11300 11400 11500 11600 11900 12000 12100 12200 12300 12400 12600 12800 12900 13000 13100 13200 13300 13400 13500 13600 13800 14000 14100 14400 99999"
+HASH_TYPES="0 10 11 12 20 21 22 23 30 40 50 60 100 101 110 111 112 120 121 122 125 130 131 132 133 140 141 150 160 200 300 400 500 600 900 1000 1100 1300 1400 1410 1411 1420 1430 1440 1441 1450 1460 1500 1600 1700 1710 1711 1720 1722 1730 1731 1740 1750 1760 1800 2100 2400 2410 2500 2600 2611 2612 2711 2811 3000 3100 3200 3710 3711 3800 3910 4010 4110 4300 4400 4500 4520 4521 4522 4700 4800 4900 5000 5100 5300 5400 5500 5600 5700 5800 6000 6100 6211 6212 6213 6221 6222 6223 6231 6232 6233 6241 6242 6243 6300 6400 6500 6600 6700 6800 6900 7000 7100 7200 7300 7400 7500 7700 7800 7900 8000 8100 8200 8300 8400 8500 8600 8700 8900 9100 9200 9300 9400 9500 9600 9700 9800 9900 10000 10100 10200 10300 10400 10500 10600 10700 10800 10900 11000 11100 11200 11300 11400 11500 11600 11900 12000 12001 12100 12200 12300 12400 12600 12700 12800 12900 13000 13100 13200 13300 13400 13500 13600 13800 14000 14100 14400 14600 14700 14800 14900 15000 15100 15200 15300 15400 15500 15600 15700 99999"
 
 #ATTACK_MODES="0 1 3 6 7"
 ATTACK_MODES="0 1 3 7"
@@ -20,17 +20,19 @@ MATCH_PASS_ONLY="2500 5300 5400 6600 6800 8200"
 
 HASHFILE_ONLY="2500"
 
-NEVER_CRACK="11600"
+NEVER_CRACK="11600 14900"
 
-SLOW_ALGOS="400 500 501 1600 1800 2100 2500 3200 5200 5800 6211 6212 6213 6221 6222 6223 6231 6232 6233 6241 6242 6243 6251 6261 6271 6281 6300 6400 6500 6600 6700 6800 7100 7200 7400 7900 8200 8800 8900 9000 9100 9200 9300 9400 9500 9600 10000 10300 10500 10700 10900 11300 11600 11900 12000 12100 12200 12300 12400 12500 12800 12900 13000 13200 13400 13600"
+SLOW_ALGOS="400 500 501 1600 1800 2100 2500 3200 5200 5800 6211 6212 6213 6221 6222 6223 6231 6232 6233 6241 6242 6243 6251 6261 6271 6281 6300 6400 6500 6600 6700 6800 7100 7200 7400 7900 8200 8800 8900 9000 9100 9200 9300 9400 9500 9600 10000 10300 10500 10700 10900 11300 11600 11900 12000 12001 12100 12200 12300 12400 12500 12700 12800 12900 13000 13200 13400 13600 14600 14700 14800 15100 15200 15300 15600 15700"
 
-OPTS="--quiet --force --potfile-disable --runtime 200 --gpu-temp-disable --weak-hash-threshold=0"
+OPTS="--quiet --force --potfile-disable --runtime 400 --gpu-temp-disable --weak-hash-threshold=0"
 
 OUTD="test_$(date +%s)"
 
 PACKAGE_CMD="7z a"
 PACKAGE_FOLDER=""
 
+EXTRACT_CMD="7z x"
+
 mask_3[0]=""
 mask_3[1]="?d"
 mask_3[2]="?d?d"
@@ -161,6 +163,65 @@ function init()
     return 0
   fi
 
+  if [[ ${hash_type} -eq 14600 ]]; then
+
+    luks_tests_folder="${TDIR}/luks_tests/"
+
+    if [ ! -d "${luks_tests_folder}" ]; then
+      mkdir -p "${luks_tests_folder}"
+    fi
+
+    luks_first_test_file="${luks_tests_folder}/hashcat_ripemd160_aes_cbc-essiv_128.luks"
+
+    if [ ! -f "${luks_first_test_file}" ]; then
+      luks_tests="hashcat_luks_testfiles.7z"
+      luks_tests_url="https://hashcat.net/misc/example_hashes/${luks_tests}"
+
+      cd ${TDIR}
+
+      # if the file already exists, but was not successfully extracted, we assume it's a broken
+      # downloaded file and therefore it should be deleted
+
+      if [ -f "${luks_tests}" ]; then
+        rm -f "${luks_tests}"
+      fi
+
+      echo ""
+      echo "ATTENTION: the luks test files (for -m ${hash_type}) are currently missing on your system."
+      echo "They will be fetched from ${luks_tests_url}"
+      echo "Note: this needs to be done only once and could take a little bit to download/extract."
+      echo "These luks test files are not shipped directly with hashcat because the file sizes are"
+      echo "particularily large and therefore a bandwidth burner for users who do not run these tests."
+      echo ""
+
+      # download:
+
+      if ! wget -q "${luks_tests_url}" &> /dev/null; then
+        cd - >/dev/null
+        echo "ERROR: Could not fetch the luks test files from this url: ${luks_tests_url}"
+        exit 1
+      fi
+
+      # extract:
+
+      ${EXTRACT_CMD} "${luks_tests}" &> /dev/null
+
+      # cleanup:
+
+      rm -f "${luks_tests}"
+      cd - >/dev/null
+
+      # just to be very sure, check again that (one of) the files now exist:
+
+      if [ ! -f "${luks_first_test_file}" ]; then
+        echo "ERROR: downloading and extracting ${luks_tests} into ${luks_tests_folder} did not complete successfully"
+        exit 1
+      fi
+    fi
+
+    return 0
+  fi
+
   # create list of password and hashes of same type
   grep " ${hash_type} '" ${OUTD}/all.sh > ${OUTD}/${hash_type}.sh 2>/dev/null
 
@@ -183,22 +244,14 @@ function init()
 
   min_len=0
 
-  if [ "${hash_type}" -eq 2500 ]; then
-
+  if   [ "${hash_type}" -eq  2500 ]; then
     min_len=7 # means length 8, since we start with 0
-
-  fi
-
-  if [ "${hash_type}" -eq 14000 ]; then
-
+  elif [ "${hash_type}" -eq 14000 ]; then
     min_len=7
-
-  fi
-
-  if [ "${hash_type}" -eq 14100 ]; then
-
+  elif [ "${hash_type}" -eq 14100 ]; then
     min_len=23
-
+  elif [ "${hash_type}" -eq 14900 ]; then
+    min_len=9
   fi
 
   while read -u 9 pass; do
@@ -239,22 +292,14 @@ function init()
 
   min_len=0
 
-  if [ "${hash_type}" -eq 2500 ]; then
-
+  if   [ "${hash_type}" -eq  2500 ]; then
     min_len=7 # means length 8, since we start with 0
-
-  fi
-
-  if [ "${hash_type}" -eq 14000 ]; then
-
+  elif [ "${hash_type}" -eq 14000 ]; then
     min_len=7
-
-  fi
-
-  if [ "${hash_type}" -eq 14100 ]; then
-
+  elif [ "${hash_type}" -eq 14100 ]; then
     min_len=23
-
+  elif [ "${hash_type}" -eq 14900 ]; then
+    min_len=9
   fi
 
   # generate multiple pass/hash foreach len (2 to 8)
@@ -312,7 +357,7 @@ function status()
         fi
 
         ;;
-      2)
+      4)
         echo "timeout reached, cmdline : ${CMD}" &>> ${OUTD}/logfull.txt
         ((e_to++))
 
@@ -625,26 +670,28 @@ function attack_1()
 
     offset=14
 
-    if   [ ${hash_type} -eq 2500 ]; then
+    if   [ ${hash_type} -eq  2500 ]; then
       offset=7
-    elif [ ${hash_type} -eq 5800 ]; then
+    elif [ ${hash_type} -eq  5800 ]; then
       offset=6
-    elif [ ${hash_type} -eq 3000 ]; then
+    elif [ ${hash_type} -eq  3000 ]; then
       offset=6
-    elif [ ${hash_type} -eq 2100 ]; then
+    elif [ ${hash_type} -eq  2100 ]; then
       offset=11
-    elif [ ${hash_type} -eq 1500 ]; then
+    elif [ ${hash_type} -eq  1500 ]; then
       offset=7
-    elif [ ${hash_type} -eq 7700 ]; then
+    elif [ ${hash_type} -eq  7700 ]; then
       offset=7
-    elif [ ${hash_type} -eq 8500 ]; then
+    elif [ ${hash_type} -eq  8500 ]; then
       offset=7
     elif [ ${hash_type} -eq 14000 ]; then
       offset=7
     elif [ ${hash_type} -eq 14100 ]; then
       offset=23
+    elif [ ${hash_type} -eq 14900 ]; then
+      offset=9
     fi
- 
+
     hash_file=${OUTD}/${hash_type}_multihash_combi.txt
 
     tail -n ${offset} ${OUTD}/${hash_type}_hashes.txt > ${hash_file}
@@ -753,25 +800,18 @@ function attack_3()
 
     # some algos have a minimum password length
 
-    if [ "${hash_type}" -eq 2500 ];then
-
+    if   [ "${hash_type}" -eq  2500 ]; then
       mask_offset=7
       max=7
-
-    fi
-
-    if [ "${hash_type}" -eq 14000 ]; then
-
+    elif [ "${hash_type}" -eq 14000 ]; then
       mask_offset=7
       max=7
-
-    fi
-
-    if [ "${hash_type}" -eq 14100 ]; then
-
+    elif [ "${hash_type}" -eq 14100 ]; then
       mask_offset=23
       max=23
-  
+    elif [ "${hash_type}" -eq 14900 ]; then
+      mask_offset=9
+      max=9
     fi
 
     i=1
@@ -894,25 +934,18 @@ function attack_3()
 
     increment_min=1
 
-    if [ "${hash_type}" -eq 2500 ]; then
-
+    if   [ "${hash_type}" -eq  2500 ]; then
       increment_min=8
       increment_max=9
-
-    fi
-
-    if [ "${hash_type}" -eq 14000 ]; then
-
+    elif [ "${hash_type}" -eq 14000 ]; then
       increment_min=8
       increment_max=8
-
-    fi
-
-    if [ "${hash_type}" -eq 14100 ]; then
-
+    elif [ "${hash_type}" -eq 14100 ]; then
       increment_min=24
       increment_max=24
-
+    elif [ "${hash_type}" -eq 14900 ]; then
+      increment_min=10
+      increment_max=10
     fi
 
     hash_file=${OUTD}/${hash_type}_multihash_bruteforce.txt
@@ -936,7 +969,13 @@ function attack_3()
 
     fi
 
-    mask=${mask_3[8]}
+    mask_pos=8
+
+    if [ "${increment_min}" -gt ${mask_pos} ]; then
+      mask_pos=${increment_min}
+    fi
+
+    mask=${mask_3[${mask_pos}]}
     custom_charsets=""
 
     # modify "default" mask if needed (and set custom charset to reduce keyspace)
@@ -1109,22 +1148,14 @@ function attack_6()
 
     max=8
 
-    if [ "${hash_type}" -eq 2500 ]; then
-
+    if   [ "${hash_type}" -eq  2500 ]; then
       max=6
-
-    fi
-
-    if [ "${hash_type}" -eq 14000 ]; then
-
+    elif [ "${hash_type}" -eq 14000 ]; then
       max=6
-
-    fi
-
-    if [ "${hash_type}" -eq 14100 ]; then
-
+    elif [ "${hash_type}" -eq 14100 ]; then
+      max=6
+    elif [ "${hash_type}" -eq 14900 ]; then
       max=6
-
     fi
 
     while read -u 9 hash; do
@@ -1218,18 +1249,20 @@ function attack_6()
 
     max=9
 
-    if   [ ${hash_type} -eq 2500 ]; then
+    if   [ ${hash_type} -eq  2500 ]; then
       max=5
-    elif [ ${hash_type} -eq 3000 ]; then
+    elif [ ${hash_type} -eq  3000 ]; then
       max=8
-    elif [ ${hash_type} -eq 7700 ]; then
+    elif [ ${hash_type} -eq  7700 ]; then
       max=8
-    elif [ ${hash_type} -eq 8500 ]; then
+    elif [ ${hash_type} -eq  8500 ]; then
       max=8
     elif [ ${hash_type} -eq 14000 ]; then
       max=5
     elif [ ${hash_type} -eq 14100 ]; then
       max=5
+    elif [ ${hash_type} -eq 14900 ]; then
+      max=5
     fi
 
     if ! contains ${hash_type} ${TIMEOUT_ALGOS}; then
@@ -1349,22 +1382,14 @@ function attack_7()
 
     max=8
 
-    if [ "${hash_type}" -eq 2500 ]; then
-
+    if   [ "${hash_type}" -eq  2500 ]; then
       max=5
-
-    fi
-
-    if [ "${hash_type}" -eq 14000 ]; then
-
+    elif [ "${hash_type}" -eq 14000 ]; then
       max=5
-
-    fi
-
-    if [ "${hash_type}" -eq 14100 ]; then
-
+    elif [ "${hash_type}" -eq 14100 ]; then
+      max=5
+    elif [ "${hash_type}" -eq 14900 ]; then
       max=5
-
     fi
 
     i=1
@@ -1476,18 +1501,20 @@ function attack_7()
 
     max=9
 
-    if   [ ${hash_type} -eq 2500 ]; then
+    if   [ ${hash_type} -eq  2500 ]; then
       max=5
-    elif [ ${hash_type} -eq 3000 ]; then
+    elif [ ${hash_type} -eq  3000 ]; then
       max=8
-    elif [ ${hash_type} -eq 7700 ]; then
+    elif [ ${hash_type} -eq  7700 ]; then
       max=8
-    elif [ ${hash_type} -eq 8500 ]; then
+    elif [ ${hash_type} -eq  8500 ]; then
       max=8
     elif [ ${hash_type} -eq 14000 ]; then
       max=5
     elif [ ${hash_type} -eq 14100 ]; then
       max=5
+    elif [ ${hash_type} -eq 14900 ]; then
+      max=5
     fi
 
     if ! contains ${hash_type} ${TIMEOUT_ALGOS}; then
@@ -1797,6 +1824,136 @@ function truecrypt_test()
   fi
 }
 
+function luks_test()
+{
+  hashType=$1
+  attackType=$2
+
+  # if -m all was set let us default to -a 3 only. You could specify the attack type directly, e.g. -m 0
+  # the problem with defaulting to all=0,1,3,6,7 is that it could take way too long
+
+  if [ "${attackType}" -eq 65535 ]; then
+    attackType=3
+  fi
+
+  #LUKS_HASHES="sha1 sha256 sha512 ripemd160 whirlpool"
+  LUKS_HASHES="sha1 sha256 sha512 ripemd160"
+  LUKS_CIPHERS="aes serpent twofish"
+  LUKS_MODES="cbc-essiv cbc-plain64 xts-plain64"
+  LUKS_KEYSIZES="128 256 512"
+
+  LUKS_PASSWORD=$(cat "${TDIR}/luks_tests/pw")
+
+  for luks_h in ${LUKS_HASHES}; do
+    for luks_c in ${LUKS_CIPHERS}; do
+      for luks_m in ${LUKS_MODES}; do
+        for luks_k in ${LUKS_KEYSIZES}; do
+
+          CMD=""
+
+          # filter out not supported combinations:
+
+          case "${luks_k}" in
+            128)
+              case "${luks_m}" in
+                cbc-essiv|cbc-plain64)
+                ;;
+                *)
+                  continue
+                ;;
+              esac
+            ;;
+            256)
+              case "${luks_m}" in
+                cbc-essiv|cbc-plain64|xts-plain64)
+                ;;
+                *)
+                  continue
+                ;;
+              esac
+            ;;
+            512)
+              case "${luks_m}" in
+                xts-plain64)
+                ;;
+                *)
+                  continue
+                ;;
+              esac
+            ;;
+          esac
+
+          luks_mode="${luks_h}-${luks_c}-${luks_m}-${luks_k}"
+          luks_file="${TDIR}/luks_tests/hashcat_${luks_h}_${luks_c}_${luks_m}_${luks_k}.luks"
+          luks_main_mask="?l"
+          luks_mask="${luks_main_mask}"
+
+          # for combination or hybrid attacks
+          luks_pass_part_file1="${OUTD}/${hashType}_dict1"
+          luks_pass_part_file2="${OUTD}/${hashType}_dict2"
+
+          case $attackType in
+            0)
+              CMD="./${BIN} ${OPTS} -a 0 -m ${hashType} ${luks_file} ${TDIR}/luks_tests/pw"
+              ;;
+            1)
+              luks_pass_part1_len=$((${#LUKS_PASSWORD} / 2))
+              luks_pass_part2_start=$((${luks_pass_part1_len} + 1))
+
+              echo "${LUKS_PASSWORD}" | cut -c-${luks_pass_part1_len} > "${luks_pass_part_file1}"
+              echo "${LUKS_PASSWORD}" | cut -c${luks_pass_part2_start}- > "${luks_pass_part_file2}"
+
+              CMD="./${BIN} ${OPTS} -a 6 -m ${hashType} ${luks_file} ${luks_pass_part_file1} ${luks_pass_part_file2}"
+              ;;
+            3)
+              luks_mask_fixed_len=$((${#LUKS_PASSWORD} - 1))
+
+              luks_mask="$(echo "${LUKS_PASSWORD}" | cut -c-${luks_mask_fixed_len})"
+              luks_mask="${luks_mask}${luks_main_mask}"
+
+              CMD="./${BIN} ${OPTS} -a 3 -m ${hashType} ${luks_file} ${luks_mask}"
+              ;;
+            6)
+              luks_pass_part1_len=$((${#LUKS_PASSWORD} - 1))
+
+              echo "${LUKS_PASSWORD}" | cut -c-${luks_pass_part1_len} > "${luks_pass_part_file1}"
+
+              CMD="./${BIN} ${OPTS} -a 6 -m ${hashType} ${luks_file} ${luks_pass_part_file1} ${luks_mask}"
+              ;;
+            7)
+              echo "${LUKS_PASSWORD}" | cut -c2- > "${luks_pass_part_file1}"
+
+              CMD="./${BIN} ${OPTS} -a 7 -m ${hashType} ${luks_file} ${luks_mask} ${luks_pass_part_file1}"
+              ;;
+          esac
+
+          if [ -n "${CMD}" ]; then
+            echo "> Testing hash type ${hashType} with attack mode ${attackType}, markov ${MARKOV}, single hash, Device-Type ${TYPE}, vector-width ${VECTOR}, luksMode ${luks_mode}" &>> ${OUTD}/logfull.txt
+
+            output=$(${CMD} 2>&1)
+            ret=${?}
+
+            echo "${output}" >> ${OUTD}/logfull.txt
+
+            cnt=1
+            e_nf=0
+            msg="OK"
+
+            if [ ${ret} -ne 0 ]; then
+              e_nf=1
+              msg="Error"
+            fi
+
+            echo "[ ${OUTD} ] [ Type ${hash_type}, Attack ${attackType}, Mode single, Device-Type ${TYPE}, Vector-Width ${VECTOR}, luksMode ${luks_mode} ] > $msg : ${e_nf}/${cnt} not found"
+
+            status ${ret}
+          fi
+        done
+      done
+    done
+  done
+}
+
 function usage()
 {
 cat << EOF
@@ -2045,8 +2202,10 @@ if [ "${PACKAGE}" -eq 0 -o -z "${PACKAGE_FOLDER}" ]; then
     # generate random test entry
     if [ ${HT} -eq 65535 ]; then
       perl tools/test.pl single > ${OUTD}/all.sh
-    elif [[ ${HT} -lt 6211 ]] || [[ ${HT} -gt 6243 ]]; then
-      perl tools/test.pl single ${HT} > ${OUTD}/all.sh
+    elif [[ ${HT} -ne 14600 ]]; then
+      if [[ ${HT} -lt  6211 ]] || [[ ${HT} -gt 6243 ]]; then
+        perl tools/test.pl single ${HT} > ${OUTD}/all.sh
+      fi
     fi
 
   else
@@ -2118,6 +2277,9 @@ if [ "${PACKAGE}" -eq 0 -o -z "${PACKAGE_FOLDER}" ]; then
               truecrypt_test ${hash_type} 0
               truecrypt_test ${hash_type} 1
               truecrypt_test ${hash_type} 2
+            elif [[ ${hash_type} -eq 14600 ]]; then
+              # run luks tests
+              luks_test ${hash_type} ${ATTACK}
             else
               # run attack mode 0 (stdin)
               if [[ ${ATTACK} -eq 65535 ]] || [[ ${ATTACK} -eq 0 ]]; then attack_0; fi
@@ -2203,7 +2365,16 @@ if [ "${PACKAGE}" -eq 1 ]; then
 
   # for convenience: 'run package' is default action for packaged test.sh ( + add other defaults too )
 
-  sed -i -e 's/^\(PACKAGE_FOLDER\)=""/\1="$( echo "${BASH_SOURCE[0]}" | sed \"s!test.sh\\$!!\" )"/' \
+  SED_IN_PLACE='-i'
+
+  UNAME=$(uname -s)
+
+  # of course OSX requires us to implement a special case (sed -i "" for the backup file)
+  if [ "${UNAME}" == "Darwin" ] ; then
+    SED_IN_PLACE='-i ""'
+  fi
+
+  sed "${SED_IN_PLACE}" -e 's/^\(PACKAGE_FOLDER\)=""/\1="$( echo "${BASH_SOURCE[0]}" | sed \"s!test.sh\\$!!\" )"/' \
     -e "s/^\(HT\)=0/\1=${HT}/" \
     -e "s/^\(MODE\)=0/\1=${MODE}/" \
     -e "s/^\(ATTACK\)=0/\1=${ATTACK}/" \
